unit UnitDocumentMethods;

interface

uses
  Windows
  , Messages
  , Dialogs
  , SysUtils
  , Classes
  , Forms
  , ShellAPI
  , contnrs
  , XmlIntf
  ,IdHTTP
  ,db
  ;

type
  EExecError = class(Exception);

  TOpenDoc = class
    FShellExecInfo: TShellExecuteInfo;
    FFilePath: String;
  Private
    function FindWindowAndClose : Boolean; // (AFilePath : String);
  end;

  TOpenDocList = class(TObjectList)
  public
    procedure CloseMyDocuments();
    function OpenDocument(DocPath: String): Boolean;
  end;

implementation

{============================================================================
  OpenDocument
  This function opens the document using the default application associated
  by Windows to the file extent.  The function provides tracking of the
  viewer application processes to allow CAPRI to close the applications and
  delete the temporary files when the VVA Retrieval window closes.

  This code was derived from the similar functionality in the ExecuteApp
  function in the ReflectionWrapper.pas unit developed by Mike Reisz.
 ============================================================================}
procedure TOpenDocList.CloseMyDocuments();
var
  I: Integer;
begin
if assigned(self) then
 if (Self.Count > 0) then
  begin
    //Terminate viewer processes to free any file locks
    for I := 0 to Count - 1 do
    begin
      if (TOpenDoc(Self[I]).FShellExecInfo.hProcess <> 0) then
      begin
        TerminateProcess(TOpenDoc(Self[I]).FShellExecInfo.hProcess, 0);
      end else begin
        TOpenDoc(Self[I]).FindwindowAndClose;
      end;
    end;

    //wait 1 second to allow viewer processes to stop before trying to delete files
    Sleep(1000);

    //Delete temporary files
    for I := Count - 1 downto 0 do
    begin
      if FileExists(TOpenDoc(Self[I]).FFilePath) then
        DeleteFile(TOpenDoc(Self[I]).FFilePath);
      Self.Remove(Self[I]);
    end;
    Self.Pack;
  end;
end;

function TOpenDocList.OpenDocument(DocPath: String): Boolean;
var
  lExitStatus: DWORD;
  lSystemMessage: string[255];
  lErrorMessage: string;
  lLength: integer;
  anOpenDoc: TOpenDoc;
begin
  lExitStatus := 0;
  anOpenDoc := TOpenDoc.Create;
  anOpenDoc.FFilePath := AnsiString(docPath);
  try
    // Initialize the ShellExecInfo record
    FillChar(anOpenDoc.FShellExecInfo, SizeOf(anOpenDoc.FShellExecInfo), 0);
    anOpenDoc.FShellExecInfo.cbSize := SizeOf(anOpenDoc.FShellExecInfo);
    // By setting fMask to SEE_MASK_NOCLOSEPROCESS, it will cause a handle to
    // the process to be set in the hProcess member.  We are responsible for
    // closing that process handle
    anOpenDoc.FShellExecInfo.fMask := SEE_MASK_NOCLOSEPROCESS;
    anOpenDoc.FShellExecInfo.Wnd := Application.Handle;
    anOpenDoc.FShellExecInfo.lpFile := PChar(DocPath);
    anOpenDoc.FShellExecInfo.lpParameters := nil;
    anOpenDoc.FShellExecInfo.lpDirectory := nil;
    anOpenDoc.FShellExecInfo.nShow := SW_SHOWNORMAL;
    // Execute the app
    if not(ShellExecuteEx(@anOpenDoc.FShellExecInfo)) then
    begin
      lExitStatus := GetLastError();
      raise EExecError.Create('The file ' + DocPath + ' failed to open');
    end
    else
    begin
      //OpenDocs.Add(anOpenDoc);
      //Self.Add(anOpenDoc);
      //Result := TRUE;
      Self.Add(anOpenDoc);
      Result:=True;

    end;
  except
    on E:Exception do
    begin
      Result := FALSE;
      lLength := FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nil, lExitStatus, 0, @lSystemMessage[1], 255, nil);
      SetLength(lSystemMessage, lLength);
      lErrorMessage := 'Exit status: ' + IntToStr(lExitStatus) + ', ' +
          lSystemMessage + #10 + 'Exception: ' + E.Message;
      MessageBox(Application.Handle, PAnsiChar(lErrorMessage), 'Error in OpenDoc', MB_ICONERROR);
      CloseHandle(anOpenDoc.FShellExecInfo.hProcess);
    end;
  end;
end;


procedure KillProcessByHwnd(hWindowHandle: integer);
var
  hprocessID: INTEGER;
  processHandle: THandle;
  DWResult: DWORD;
  lExitStatus: DWORD;
begin
  try
    Application.ProcessMessages;
    SendMessageTimeout( hWindowHandle, WM_CLOSE, 0, 0
                      , SMTO_ABORTIFHUNG or SMTO_NORMAL, 5000, DWResult);
    lExitStatus := GetLastError();

    if (lExitStatus = ERROR_TIMEOUT) then
    begin
      sleep(2000);
      SendMessageTimeout( hWindowHandle, WM_CLOSE, 0, 0
                        , SMTO_ABORTIFHUNG or SMTO_NORMAL, 5000, DWResult);
      Application.ProcessMessages;
    end;

    if isWindow(hWindowHandle) then begin
      PostMessage(hWindowHandle, WM_QUIT, 0, 0);  // send it so it will quit soon

      { Get the process identifier for the window}
      GetWindowThreadProcessID(hWindowHandle, @hprocessID);
      if hprocessID <> 0 then begin
        { Get the process handle }
        processHandle := OpenProcess(PROCESS_TERMINATE or PROCESS_QUERY_INFORMATION,
          False, hprocessID);
        if processHandle <> 0 then begin
          { Terminate the process }
          TerminateProcess(processHandle, 0);
          CloseHandle(ProcessHandle);
        end;
      end;
    end;
  except
    on E:Exception do
    begin
      // nothing
    end;
  end;
end;

function FindWinToClose(hHwnd: HWND; lParam: LPARAM): boolean; stdcall;
var
  //pPid: Pointer;
  title, FileName: string;
begin
  result := true;
  //if the returned value is null the
  //callback has failed, so set to false and exit.
  if (hHwnd = 0) then begin
    result := false;
    exit;
  end;

  SetLength(title, 255);
  // LMS some GetwindowText could return 0 length..  SetLength(title, GetWindowText(hHwnd, PChar(title), Length(title)));
  SetLength(title, SendMessage(hHwnd, WM_GETTEXT, Length(title), Integer(PChar(title))));

  title := AnsiUpperCase(title);

  //SetString(aReturnTextAnsi,PWideChar(aReturnTextP),aRetBufSize div 2); // widechar gets div 2
  FileName := String(Pointer(lParam));
  if (System.Pos(FileName, title) <> 0) then
  begin
    KillProcessByHwnd(hHwnd);
    result := false;  // no more enumeration needed // isWindow(hHwnd); // true;
  end;
end;

{ TOpenDoc }

function FindWindowByTitleAndClose(WindowTitleToClose : string) : Boolean; // (AFilePath: String);
var
  lp: Pointer;
  FileName : string;
begin
  FileName := AnsiUpperCase(WindowTitleToClose);
  lp := Pointer(FileName);
  Result := EnumWindows(@FindWinToClose, LPARAM(lp));
  (*if EnumWindows(@FindWinToClose, lp) = false then
    ShowMessage('Error: Could not obtain process window hook from system.');*)
end;

function TOpenDoc.FindWindowAndClose : Boolean; // (AFilePath: String);
begin
  Result := FindWindowByTitleAndClose(ExtractFileName(FFilePath));
  (*FileName := AnsiUpperCase(ExtractFileName(FFilePath));
  lp := Pointer(FileName);
  Result := EnumWindows(@FindWinToClose, LPARAM(lp));
  (*if EnumWindows(@FindWinToClose, lp) = false then
    ShowMessage('Error: Could not obtain process window hook from system.');*)
end;

end.
