unit VlerDasClient;

{===============================================================================
  VlerDasClient
    This unit was created to provide client calls to
    ProvideAndRegisterDocumentSetRequest in the VLER DAS web service.  Note that
    TfVlerDasClient was originally going to be derived from TObject.  However,
    the TXMLDocument objects need a parent form to operate correctly.

  Modification History:
    patch 186 - Unit created 7/2013 -MER
    patch 186 - CodeCR540 12/6/13 JRL, Added idSSLIOHandlerSocketOpenSSL2 to
                implement 1-way SSL to the proxy server
 ==============================================================================}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, xmldom, XMLIntf, msxmldom, XMLDoc, IdBaseComponent, IdComponent,
  IdTCPConnection, IdTCPClient, IdHTTP, IdCoder, IdCoder3to4, IdCoderMIME,
  StdCtrls, ExtCtrls, Main, IdIOHandler, IdIOHandlerSocket, IdIOHandlerStack,
  IdSSL, IdSSLOpenSSL
  ,idCookieManager  // CodeCR540 JRL 12/12/13
  ,idCookie         // CodeCR540 JRL 12/12/13
  , VlerDasIdHTTP
  ;

const
  NS_URN = 'urn:ihe:iti:xds-b:2007';
  NS_URN1 = 'urn:oasis:names:tc:ebxml-regrep:xsd:lcm:3.0';
  NS_URN2 = 'urn:oasis:names:tc:ebxml-regrep:xsd:rs:3.0';
  NS_URN3 = 'urn:oasis:names:tc:ebxml-regrep:xsd:rim:3.0';
  NS_NS8  = 'urn:ihe:iti:xdr:2007';
  FN_SOAP_RESPONSE = 'VlerDasClientResponse.xml';
  FN_SOAP_REQUEST = 'VlerDasClientRequest.xml';

type
  TfVlerDasClient = class(TForm)
    xdRequest: TXMLDocument;
    xdResponse: TXMLDocument;
    // idhClient: TIdHTTP;           // DONE: LMS moved functionality
    TransmittingDBQ: TLabel;
    DBQName: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FDBQ: string;
    //FIdSSLIOHandlerSocketOpenSSL: TIdSSLIOHandlerSocketOpenSSL;
    //function  GetStringGUID: string;         //  DONE: LMS refactored
    function  GetResponseString(xDoc: IXMLDocument): string;
    procedure PrepareSOAP(sDocId: string);
    function  SendRequest: string;
    //function  Base64Encode(theText : string): string;   // DONE: CodeCR540 - LMS - utility function.. moved out of class.
  public
    idhClient: TIdHTTPVler;
    property DBQ: string read FDBQ write FDBQ; // this property is provided for
      // convenience of development testing.  Future development may desire an
      // inteface where the DBQ is passed as a string as opposed to TXMLDocument
    procedure SetDBQName(theDBQName: String);
    function ProvideAndRegisterDocumentSetRequest(xdDBQ: IXMLDocument; sDocId: string): string;
    //procedure SetCookieManager(CookieMgr : TIdCookieManager);                   //CodeCR540 JRL 12/12/13      // Removed : LMS
  end;

implementation

{$R *.dfm}


uses untMiscMthds
  //, IdURI
  //, StrUtils
  //, untBrkrMthds
  ;


const
  HTTPHeaderAcceptXML = 'application/xml';
  HTTPContentType = 'text/xml';
  HTTPContentEncoding = 'utf-8';


(*
procedure TIdHTTPvler.SetupHTTPSrvHeader;
 // CodeCR??? LMS - New method to consolidate all access to VLER
begin
  Request.Clear;
  Request.Accept := HTTPHeaderAcceptXML; //'application/xml' - DEFAULT is JASON
  Request.ContentType := HTTPContentType;
  Request.ContentEncoding := HTTPContentEncoding;
  Request.CustomHeaders.Clear;
  Request.CustomHeaders.Add('authToken:' + VlerDasToken);
end;
*)

(*
{===============================================================================
  Base64Encode
    Base64Encode is a function that encodes an input string into base64.  The
      TIdEncoderMime results are not limited to 76 char lines.  Testing against
      the WS indicates that this is required.  As a result, the lines are
      terminated at 76 characters

  Parameters
    theText:
      A string representing the data to be encoded

  Return values
    Base64Encode returns a string representing the encoded result

  Modification History:
    patch 186 - Method created 7/2013 -MER
 ==============================================================================}
class function TfVlerDasClient.Base64Encode(theText : string): string;
var
  lEncoder: TIdEncoderMime;
  lNdx:     integer;
  lString1: string;
  lString2: string;
begin
  lString2 := '';
  lEncoder := TIdEncoderMime.Create(nil);
  try
    lString1 := lEncoder.Encode(theText);
    // For the encoding to be accepted by the web service operation we must
    // force the line length to 76 characters terminating each line with x0D x0A
    for lNdx := 0 to (Length(lString1) - 1) do begin
      if lNdx MOD 76 = 0 then
        lString2 := lString2 + #13#10;
      lString2 := lString2 + lString1[lNdx + 1];
    end;
    Result := lString2;
  finally
    lEncoder.Free;
  end
end;
*)

(*    DONE :  LMS Refactored
{===============================================================================
  GetStringGUID
    GetStringGUID is a function that generates a formatted GUID

  Return values
    GetStringGUID returns a string representing the formatted GUID

  Modification History:
    patch 186 - Method created 7/2013 -MER
 ==============================================================================}
function TfVlerDasClient.GetStringGUID: string;
var
  lGUID:            TGUID;
  lGUIDStr:         string;
begin
  CreateGUID(lGUID);
  lGUIDStr := GUIDToString(lGUID);
  lGUIDStr := StringReplace(lGUIDStr, '{', '', [rfReplaceAll, rfIgnoreCase]);
  lGUIDStr := StringReplace(lGUIDStr, '}', '', [rfReplaceAll, rfIgnoreCase]);
  result := lGUIDStr;
end;
*)

{===============================================================================
  GetResponseString
    GetResponseString is a function that parses the response SOAP and returns
      the response message

  Parameters
    xDoc:
      An IXMLDocument that contains the response SOAP

  Return values
    GetResponseString returns a string representing the response message

  Modification History:
    patch 186 - Method created 7/2013 -MER
    patch 186 - Added try except loop and exception result JRL 12/6/13
 ==============================================================================}
procedure TfVlerDasClient.FormCreate(Sender: TObject);
begin
  idhClient := TIdHTTPVler.Create(self,frmMain.RPCBroker1);     // setting this back to preferred owner, since create issue is resolved.
end;

procedure TfVlerDasClient.FormDestroy(Sender: TObject);
begin
  // idhClient.Free;
end;

function TfVlerDasClient.GetResponseString(xDoc: IXMLDocument): string;
var
  lNode:            IXMLNode;
begin
  result := '';
  try
    xDoc.Active := TRUE;
    lNode := xDoc.ChildNodes['S:Envelope'];
    lNode := lNode.ChildNodes['S:Body'];
    lNode := lNode.ChildNodes.FindNode('Acknowledgement', NS_NS8);
    // For some unknown reason, one can't access the node by name, so it is here
    // referenced by index
    lNode := lNode.ChildNodes[0];
    if lNode <> nil then
      result := lNode.Text;
  except
    on e:exception do
       result := xDoc.XML.Text;   // Added for proxy server response
  end;
end;

{===============================================================================
  PrepareSOAP
    PrepareSOAP is a procedure that modifies a stored SOAP request to contain
      the particular details of a given call.  The base SOAP request is stored
      in xdRequest.XML.  Three items need to be modified to make the correct
      SOAP request, the SubmitObjectsRequest id, the Document id and the
      Document itself (which will contain the DBQ).  Note that in verbal
      discussions with the VLER DAS team, they indicated they would be changing
      to include a requirement that the SOAP Document id GUID match the id
      within the DBQ.  As a result, this GUID is passed in with the call to
      ProvideAndRegisterDocumentSetRequest

  Parameters
    sDocId:
      A string containg the DBQ Document id GUID

  Modification History:
    patch 186 - Method created 7/2013 -MER
 ==============================================================================}
procedure TfVlerDasClient.PrepareSOAP(sDocId: string);
var
  lNode:            IXMLNode;
  lAttributeNode:   IXMLNode;
begin
  // navigate through nodes to SubmitObjectsRequest
  xdRequest.Active := TRUE;
  lNode := xdRequest.ChildNodes['env:Envelope'];
  lNode := lNode.ChildNodes['env:Body'];
  lNode := lNode.ChildNodes.FindNode('ProvideAndRegisterDocumentSetRequest', NS_URN);
  lNode := lNode.ChildNodes.FindNode('SubmitObjectsRequest', NS_URN1);
  // set id attribute of SubmitObjectsRequest
  lAttributeNode := lNode.AttributeNodes.FindNode('id');
  lAttributeNode.Text := GetStringGUID;
  // navigate through nodes to Document
  lNode := lNode.ParentNode;
  lNode := lNode.ChildNodes.FindNode('Document', NS_URN);
  // set id attribute of Document
  lAttributeNode := lNode.AttributeNodes.FindNode('id');
  lAttributeNode.Text := sDocId;
  // Load the DBQ into the document node
  lNode.Text := base64encode(FDBQ);
end;

{===============================================================================
  SendRequest
    SendRequest is a function that posts the request SOAP contained within
      xdRequest and returns the response message

  Return values
    SendRequest returns a string representing the response message

  Modification History:
    patch 186 - Method created 7/2013 -MER
    patch 186 Security Fix - CodeCR540 JRL 12/11/13
                             Added Authentication Token to the header
 ==============================================================================}
function TfVlerDasClient.SendRequest: string;
var
  lStream:    TStringStream;
  PostToThisURL : string;
  TheResponse : string;
begin
  lStream := TStringStream.Create('');
  try
    xdRequest.XML.SaveToStream(lStream);

    if PARAM_SAVE_VLER_WS = True then
    begin
      // save the request to a file
      if FileExists(FN_SOAP_REQUEST) then
        DeleteFile(FN_SOAP_REQUEST);
      xdRequest.SaveToFile(FN_SOAP_REQUEST);
    end;

    lStream.Position := 0;
    idhClient.InitVLERBaseRequestHeader;      // LMS - reset to known header and other values.
    //idhClient.Request.Accept := 'text/xml';   // LMS Reference  RFC (3023). "if it's unreadable to a casual reader, use application/xml, if it's readable, use text/xml".
               // Since, we're always processing the result in the app and it's not meant for human consumption directly, it should always be application/xml

    // DONE: LMS - All moved into common handling of headers. Override headers prior to making a request as needed.
    //idhClient.Request.ContentType := 'text/xml';
    //idhClient.Request.ContentEncoding := 'utf-8';
    //idhClient.Request.CustomHeaders.Clear;
    //tStr := VlerDasToken;
    //tStr := encrypt(tStr);
    //idhClient.Request.CustomHeaders.Add('authToken:' + tStr);   //CodeCR540 JRL 12/11/13
    //idhClient.ReadTimeout := 40000; // 40 seconds
    //idhClient.ConnectTimeout := 10000; // 10 seconds

    xdResponse.XML.Clear;
    // Post the request and save the response in xdResponse
    try
      if (copy(TIdHTTPVler.GetVlerDasURL(), length(TIdHTTPVler.GetVlerDasURL()) - 8, 8) = 'tService') then      // TODO: LMS temporary fix due to users having old shortcuts with additional path. These shortcuts should be removed and this code should not have to account for those in the near future.
        PostToThisURL := TIdHTTPVler.GetVlerDasURL()
      else
        PostToThisURL := TIdHTTPVler.GetVlerDasURL() + '/BTSXDRRequest_PortTypeWS/BTSXDRRequestService';

      TheResponse := idhClient.Post(PostToThisURL, lStream); // CodeCR500 JRL 8/27/13 // CodeCR566 LMS 2014-07-31 - generic URL so the proxy no longer does the full URL

      // idhClient.Response.ResponseCode;  // CodeCR 566 LMS 2014-08-07  Exposing the Response object of the managedIdHttp object for direct use. The 200 code should be used for BTS, instead of the Success text.

      xdResponse.XML.Add(TheResponse); // CodeCR500 JRL 8/27/13 // CodeCR566 generic URL so the proxy no longer does the full URL
//      xdResponse.XML.Add(idhClient.Post(VlerDasURL + '/BTSXDRRequest_PortTypeWS/BTSXDRRequestService', lStream)); // CodeCR500 JRL 8/27/13 // CodeCR566 generic URL so the proxy no longer does the full URL
//      xdResponse.XML.Add(idhClient.Post(VlerDasURL, lStream));            // CodeCR500 JRL 8/27/13
      xdResponse.Active := TRUE;
      if PARAM_SAVE_VLER_WS = True then
      begin
        // save the response to a file
        if FileExists(FN_SOAP_RESPONSE) then
          DeleteFile(FN_SOAP_RESPONSE);
        xdResponse.SaveToFile(FN_SOAP_RESPONSE);
      end;
      Result := GetResponseString(xdResponse);
    except
      on E: Exception do begin
        if E.Message <> '' then
          Result := E.Message
        else
          Result := 'CAPRI: An unknown exception occurred during the web service call.';
      end;
    end;
  finally
    lStream.Free;
  end;
end;

{===============================================================================
  ProvideAndRegisterDocumentSetRequest
    ProvideAndRegisterDocumentSetRequest is a public function that invokes
      the web service operation ProvideAndRegisterDocumentSetRequest

  Parameters
    xdDBQ:
      An TXMLDocument that contains the DBQ template to be sent
    sDocId:
       A string containing the Document id GUID of the DBQ

  Return values
    ProvideAndRegisterDocumentSetRequest returns a string representing the
      response message

  Modification History:
    patch 186 - Method created 7/2013 -MER
 ==============================================================================}
function TfVlerDasClient.ProvideAndRegisterDocumentSetRequest(xdDBQ: IXMLDocument; sDocId: string): string;
begin
  Screen.Cursor := crHourGlass;
  FDBQ := xdDBQ.XML.Text;
  PrepareSoap(sDocId);
  Result := SendRequest;
  Screen.Cursor := crDefault;
end;


procedure TfVlerDasClient.SetDBQName(theDBQName: string);
begin
  DBQName.Caption := theDBQName;
end;

//------------------------------------------------------------------------------
// Added routine to allow multiple DBQs to be sent using only one authorization
// token.  The first DBQ will validate the authorization token and subsequent
// DBQs with the same exam will be sent "as authorized".
//
// CodeCR540 JRL 12/12/13
//------------------------------------------------------------------------------
(*
procedure TfVlerDasClient.SetCookieManager(CookieMgr: TIdCookieManager);
begin
  idhClient.CookieManager := CookieMgr;
end;
*)

end.
