unit VocRehabMedicalRequest;
//------------------------------------------------------------------------------
// CodeCR347 - JRL June, 2012 - Added unit/form
// Vocational Rehab Form 28-8861
// 
// This form enables the input and/or editing of the new electronic 
// 28-8861 Vocational Rehab form. 
//------------------------------------------------------------------------------
             
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, ExtCtrls, ActnList, XPStyleActnCtrls,
  ActnMan, ToolWin, ActnCtrls, ActnMenus, Inifiles, DateUtils, CommCtrl,
  CCOWRPCBrokerCAPRI, Fmcntrls, DialogsCAPRI, Diaccess, Fmcmpnts, DiTypLib,
  Trpcb, ORCtrls, VA508AccessibilityManager;

const
   REASON_OTHER = 'OTHER';
   NO_CONSULTS_FOUND = 'No Consults Found';
   NO_ITEMS_FOUND = 'No Items Found';
type
  TfrmVRMedicalRequest = class(TForm)
    pnlRouting: TPanel;
    lblRegionalOffice: TLabel;
    pnlPreferredDate: TPanel;
    lblPreferredScheduleDateReason: TLabel;
    edtPreferredScheduleDateReason: TEdit;
    pnlComments: TPanel;
    pnlMedicalServices: TPanel;
    lblMedicalServicesRequested: TLabel;
    lblServicesOther: TLabel;
    edtServicesOther: TEdit;
    lblRehabObjectives: TLabel;
    reRehabObjectives: TRichEdit;
    btnSendRequest: TButton;
    pnlConsultsSelected: TPanel;
    lblConsultsSelected: TLabel;
    pnlRehabDate: TPanel;
    Label3: TLabel;
    edtReasonForReferral: TEdit;
    btnCancelRequest: TButton;
    FMRegionalOfficeNumberIEN: TFMEdit;
    FMRegionalOfficeNumber: TFMComboBox;
    btnPOC: TBitBtn;
    btnCloseWindow: TButton;
    btnAssignConsults: TButton;
    FMGetsVRMedServices: TFMGets;
    FMGetsVR: TFMGets;
    edtRegionalOfficeReadOnly: TEdit;
    edtSelectedConsult: TEdit;
    FMFilerVR: TFMFiler;
    FMCommentsNotes: TFMMemo;
    FMListerVocRehab: TFMLister;
    FMListerVRMedSvcReq: TFMLister;
    cbMedicalServicesRequested: TComboBox;
    edtMedicalServicesRequestedReadOnly: TEdit;
    pnlHeader: TPanel;
    edtVAClaimNumber: TEdit;
    edtPatientName: TEdit;
    edtDateSubmitted: TEdit;
    FMGetsRegionalOffice: TFMGets;
    leAnticipatedDate: TLabeledEdit;
    btnCalAnticipatedDate: TSpeedButton;
    lePreferredDate: TLabeledEdit;
    btnCalPreferredDate: TSpeedButton;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    lblCommentsNotes: TLabel;
    procedure btnSendRequestClick(Sender: TObject);
    procedure btnAssignConsultsClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure dtPreferredScheduleDateChange(Sender: TObject);
    procedure cbMedicalServicesRequestedChange(Sender: TObject);
    procedure dtRehabilitationChange(Sender: TObject);
    procedure btnCancelRequestClick(Sender: TObject);
    procedure FMRegionalOfficeNumberEnter(Sender: TObject);
    procedure btnPOCClick(Sender: TObject);
    procedure btnCloseWindowClick(Sender: TObject);
    procedure FMCommentsNotesChange(Sender: TObject);
    procedure btnCalAnticipatedDateClick(Sender: TObject);
    procedure btnCalPreferredDateClick(Sender: TObject);
    procedure leAnticipatedDateExit(Sender: TObject);
    procedure lePreferredDateExit(Sender: TObject);
  private
    { Private declarations }
    OneRegionalOffice : Boolean;
    procedure PopulateFieldValues;
    function  ShowCancelRequestButton : Boolean;
    function  AreRequiredFieldsComplete : Boolean;
    procedure GetRegionalOfficeValues;
    procedure GetMedicalServicesRequestedValues;
    function  EnableAssignConsultsButton : Boolean;
    procedure SendRecord(RecordIEN : string; CommentsOnly : Boolean);
    function  ConvertRichEditToString(reInput : TRichEdit) : string;  // works for small richedit fields
    procedure DateComplete(var StrLikeDate : string; MinDate : TDate; const NowIfError : Boolean = TRUE);
  public
    { Public declarations }
    SelectedMedicalRequestIEN : string; // Populated on main.dfm when Medical Services Request is selected
    procedure SetupNewExam;
    procedure SetupExistingExam;
    procedure ClearAndInitiateFields;
    procedure PopulateConsultSelectedField;
    procedure SetEditsToReadOnly(Value : Boolean);
  end;

var
  frmVRMedicalRequest: TfrmVRMedicalRequest;

implementation

uses Main, VocRehabAssnConsults, VocRehabCancelExam, rofinder, essoselect,
     CAPRISupport, VocRehabPOC, VAUtils, VocRehabSelectDate;

{$R *.dfm}

//------------------------------------------------------------------------------
// Form Activate
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.FormActivate(Sender: TObject);            
begin
   if VAUtils.ScreenReaderActive then
   begin
     edtPatientName.TabStop := TRUE;
     edtVAClaimNumber.TabStop := TRUE;
     edtDateSubmitted.TabStop := TRUE;
     edtPatientName.SetFocus;
   end
   else
   begin
     edtPatientName.TabStop := FALSE;
     edtVAClaimNumber.TabStop := FALSE;
     edtDateSubmitted.TabStop := FALSE;
     leAnticipatedDate.SetFocus;
   end;
   // editing an exam, set edits to editable/read-only based on medical request status
   if frmMain.VocRehab.AddExam = FALSE then  // editing
   begin
     if (frmMain.VocRehab.RequestCancelled) OR // if the request is completed or cancelled, don't allow editing
        (frmMain.VocRehab.RequestCompleted) OR  // if the consult is already assigned, don't allow editing
        (btnCancelRequest.Visible = FALSE) then
        SetEditsToReadOnly(TRUE)
     else
        SetEditsToReadOnly(FALSE);
   end
   else
   begin
      SetEditsToReadOnly(FALSE);
   end;
   
end;

//------------------------------------------------------------------------------
// Button - Point of Contact Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnPOCClick(Sender: TObject);
begin
   // Display Point of Contact information
   try
      frmVRPointOfContact := TfrmVRPointOfContact.Create(Application);
      frmVRPointOfContact.ShowModal;
   finally
     FreeAndNil(frmVRPointOfContact);
   end;
end;

//------------------------------------------------------------------------------
// Button - Assign Consults Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnAssignConsultsClick(Sender: TObject);
begin
  // This button is only visible if the logged in user has security keys for
  // "DVBA CAPRI VHA_VRECoordinator"
   try
      frmVRAssignConsults := TfrmVRAssignConsults.Create(Application);
      frmVRAssignConsults.ShowModal;
   finally
     FreeAndNil(frmVRAssignConsults);
   end;
   if edtSelectedConsult.Text = NO_CONSULTS_FOUND then
      btnAssignConsults.Visible := TRUE
   else
      btnAssignConsults.Visible := FALSE;
end;

//------------------------------------------------------------------------------
// Button - Cancel Request Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnCancelRequestClick(Sender: TObject);
begin
   // Cancel existing request for medical services.  This button is only visible
   // when editing an existing medical request and a consult hasn't been assigned 
   try
      frmVRCancelExam := TfrmVRCancelExam.Create(Application);
      frmVRCancelExam.ShowModal;
   finally
     FreeAndNil(frmVRCancelExam);
   end;
   frmMain.VocRehab.RequestCancelled := TRUE;
   SetEditsToReadOnly(TRUE);
   if btnAssignConsults.Visible then
      btnAssignConsults.Enabled := FALSE;
end;

//------------------------------------------------------------------------------
// Button - Close Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnCloseWindowClick(Sender: TObject);
begin
  Close;
end;

//------------------------------------------------------------------------------
// Button - Send Request Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnSendRequestClick(Sender: TObject);
begin
  // Anticipated Date of Rehabilitation cannot be blank
  if AreRequiredFieldsComplete = FALSE then
     Exit;
  if frmMain.VocRehab.AddExam then  // creating a new record
  begin
    SendRecord('+1,',FALSE);  // New record
    frmMain.VocRehab.SendMailManMessage('NEW');  
  end
  else // editing existing exam
  begin
    if (edtSelectedConsult.Text = '') OR (edtSelectedConsult.Text = NO_CONSULTS_FOUND) then // If no consult is assigned, update a full edited record
      SendRecord(SelectedMedicalRequestIEN,FALSE)
    else // Update comments only because consult is assigned
      SendRecord(SelectedMedicalRequestIEN,TRUE); 
  end;
  Close;
end;

//------------------------------------------------------------------------------
// Get Regional Office Information
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.GetRegionalOfficeValues;
var
  foundflag: boolean;
  x : Integer;
begin
  If essoversion = true Then
  Begin
    // Make sure user can be on current site before running a report
    foundflag := false;
    If formESSOSelect.slUserSites.Count > 0 Then  //CodeCR77
      For x := 0 To formESSOSelect.slUserSites.Count - 1 Do
        If uppercase(frmMain.piece(formESSOSelect.slUserSites[x], '^', 2)) = uppercase(frmMain.RPCBroker1.Server) Then
          foundflag := true;
    If foundflag <> true Then
    Begin
      ShowmessageCAPRI('You do not have direct access to this site.  Please use FILE | SWITCH SITES and pick a different site prior to running a report.');
      exit;
    End;
  End;
  If frmROFinder.ORListBoxSites.Items.Count = 0 Then
     frmMain.LoadInstitutions;
  FMRegionalOfficeNumber.Items.Clear;

  if frmMain.VocRehab.AddExam then
  begin
    // Set up division...
    searchforstring := UserDivision;
    FMRegionalOfficeNumberEnter(Application);
    searchforstring := '';
    If FMRegionalOfficeNumber.Text <> '' Then
    begin
      edtRegionalOfficeReadOnly.Text := FMRegionalOfficeNumber.Text;
      FMRegionalOfficeNumber.Enabled := False;
      FMRegionalOfficeNumber.Visible := FALSE;
      edtRegionalOfficeReadOnly.Visible := TRUE;
      if VAUtils.ScreenReaderActive then
        edtRegionalOfficeReadOnly.Enabled := TRUE;
    end;
  end
  else
  begin
    FMGetsRegionalOffice.IENS := SelectedMedicalRequestIEN;
    FMGetsRegionalOffice.GetAndFill;
    If FMRegionalOfficeNumber.Text <> '' Then
    begin
      FMRegionalOfficeNumber.Enabled := False;
      FMRegionalOfficeNumber.Visible := FALSE;
      edtRegionalOfficeReadOnly.Visible := TRUE;
      if VAUtils.ScreenReaderActive then
        edtRegionalOfficeReadOnly.Enabled := TRUE;
      OneRegionalOffice := FALSE;
    end
    else
      OneRegionalOffice := TRUE;
  end;
end;

//------------------------------------------------------------------------------
// Anticipated Date Exit
//------------------------------------------------------------------------------
// Validate date using DateComplete procedure.  If date is blank, allow that
// during the edit phase.  A blank date will not be allowed once the Send
// button is pressed.  
procedure TfrmVRMedicalRequest.leAnticipatedDateExit(Sender: TObject);
var
  s : string;
  SaveDate : string;
  MinDate : TDate;
begin
  s := leAnticipatedDate.Text;
  SaveDate := s;
  if frmMain.VocRehab.AddExam then // if new request, the minimum date is Tomorrow
     MinDate := Tomorrow
  else  // if editing a request, allow a previous date
     MinDate := StrToDate('01/01/1900'); 
  DateComplete(s, MinDate, FALSE);
  leAnticipatedDate.Text := s;
//  if leAnticipatedDate.Text = '' then
//  begin
//    ShowMessage(SaveDate + ' is not a valid date.');
//    leAnticipatedDate.SetFocus;
//  end;
  if copy(leAnticipatedDate.Text,1,4) = 'Min ' then
  begin
    leAnticipatedDate.Text := '';
    ShowMessage('Anticipated Date must be a future date');
    leAnticipatedDate.SetFocus;
  end;               
   
end;

//------------------------------------------------------------------------------
// Preferred Date Exit
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.lePreferredDateExit(Sender: TObject);
var
  s : string;
  SaveDate : string;
  MinDate : TDate;
begin
  s := lePreferredDate.Text;
  SaveDate := s;
  if frmMain.VocRehab.AddExam then // if new request, the minimum date is Tomorrow
     MinDate := Tomorrow
  else
     MinDate := StrToDate('01/01/1900');
  DateComplete(s, MinDate, FALSE);
  lePreferredDate.Text := s;
//  if lePreferredDate.Text = '' then
//  begin
//    ShowMessage(SaveDate + ' is not a valid date.');
//    lePreferredDate.SetFocus;
//  end;
  if copy(lePreferredDate.Text,1,4) = 'Min ' then
  begin
    lePreferredDate.Text := '';
    ShowMessage('Preferred Date must be a future date');
    lePreferredDate.SetFocus;
  end;   
  if (frmMain.VocRehab.RequestCancelled) OR
     (frmMain.VocRehab.RequestCompleted) then
  begin
    edtPreferredScheduleDateReason.ReadOnly := TRUE;
//  lblPreferredScheduleDateReason.Font.Color := clGray;
//  edtPreferredScheduleDateReason.Color := clBtnFace;
  end
  else
  begin
     if lePreferredDate.Text <> '' then
     begin
       edtPreferredScheduleDateReason.ReadOnly := FALSE;
       lblPreferredScheduleDateReason.Font.Color := clBlack;
       edtPreferredScheduleDateReason.Color := clWhite;
       edtPreferredScheduleDateReason.Font.Color := clBlack;
     end
     else
     begin
       edtPreferredScheduleDateReason.ReadOnly := TRUE;
       lblPreferredScheduleDateReason.Font.Color := clGray;
       edtPreferredScheduleDateReason.Color := clBtnFace;
     end;
  end;
end;

//------------------------------------------------------------------------------
// Comments Change
//------------------------------------------------------------------------------
// If form is in ReadOnly mode, then the SendMedicalRequest button is not 
// enabled.  If changes are made to the comments/nodes field (only field
// editable in this mode, then enable the SendMedicalRequestButton
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.FMCommentsNotesChange(Sender: TObject);
begin
  if leAnticipatedDate.ReadOnly = TRUE then // we are in ReadOnlyMode
  begin
     btnSendRequest.Enabled := TRUE;
     btnSendRequest.Visible := TRUE;
  end;
end;


//------------------------------------------------------------------------------
// Regional Office Control Enter
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.FMRegionalOfficeNumberEnter(Sender: TObject);
begin
  // code stolen from newexam.pas:  TfrmNewExam.FMRegionalOfficeNumberEnter(Sender: TObject);
  // with a few substractions for unnecessary code in this instance
  If frmROFinder.ShowModal = mrOK Then
    If frmROFinder.ORListBoxSites.ItemIndex > -1 Then
    Begin
      FMRegionalOfficeNumber.Visible := TRUE;  
      FMRegionalOfficeNumber.Text := frmMain.Piece(frmROFinder.ORListBoxSites.Items[frmROFinder.ORListBoxSites.ItemIndex], '^', 1);
      FMRegionalOfficeNumberIEN.Text := frmMain.Piece(frmROFinder.ORListBoxSites.Items[frmROFinder.ORListBoxSites.ItemIndex], '^', 2);
    End;
  // note: since this combo box is being used in a non-conventional way, it is
  // necessary for focus to pass to the next control to behave as the users are
  // accustomed
  if (ActiveControl = FMRegionalOfficeNumber) then
    SelectNext(ActiveControl as TWinControl, TRUE, TRUE);
end;

//------------------------------------------------------------------------------
// Get Medical Services Requested Values to populate cbMedicalServicesRequested
//------------------------------------------------------------------------------
// Each cbMedicalServicesRequested entry contains a string value and an object.
// The object contains the integer IEN number for the Medical Service Requested.
// The customer requested that the combobox display the data in the order shown,
// so the data was input into the database in the desired order and the combobox
// is sorted by IEN number preserving the requested order.
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.GetMedicalServicesRequestedValues;

  function SortByIEN(List: TStringList; Index1,Index2: Integer): Integer;
  var
    IEN1, IEN2 : Integer;
  begin
    Result := 0;
    IEN1 := Integer(List.Objects[Index1]);
    IEN2 := Integer(List.Objects[Index2]);
    if IEN1 > IEN2 then
      Result := 1;
    if IEN1 = IEN2 then
      Result := 0;
    if IEN1 < IEN2 then
      Result := -1;
  end; 
  
var
  x : Integer;
  buffer : string;
  slNames : TStringList;
begin     
  cbMedicalServicesRequested.Items.Clear;
  FMListerVRMedSvcReq.Screen := '';
  FMListerVRMedSvcReq.FMIndex := '';
  FMListerVRMedSvcReq.DisplayFields.Clear;
  FMListerVRMedSvcReq.FieldNumbers.Clear;
  FMListerVRMedSvcReq.FileNumber := '396.918';
  FMListerVRMedSvcReq.DisplayFields.Add('.01');
  FMListerVRMedSvcReq.FieldNumbers.Add('.01');
  FMListerVRMedSvcReq.GetList(nil);
  if FMListerVRMedSvcReq.Results.Count < 1 then
  begin
     cbMedicalServicesRequested.Items.Add(NO_ITEMS_FOUND);
     Exit;
  end;
  // Get and parse the returned records  
  for x := 0 to FMListerVRMedSvcReq.Results.Count - 1 do
  begin
    buffer := FMListerVRMedSvcReq.GetRecord(FMListerVRMedSvcReq.Results[x]).GetField('.01').FMDBExternal;
    cbMedicalServicesRequested.AddItem(buffer,TObject(StrToInt(FMListerVRMedSvcReq.Results[x]))); // add item stores the record IEN in the obejct
  end;
  // sort combobox on IEN object using a customsort.  Combobox doesn't have
  // a customsort method so assign the combobox items to a TStringlist.
  slNames := TStringList.Create;
  cbMedicalServicesRequested.Items.BeginUpdate;
  cbMedicalServicesRequested.Sorted := FALSE;
  try
    slNames.Assign(cbMedicalServicesRequested.Items);
    slNames.CustomSort(@SortByIEN);
    cbMedicalServicesRequested.Items.Assign(slNames);
  finally
    FreeAndNil(slNames);
    cbMedicalServicesRequested.Items.EndUpdate;
  end;
end;


//------------------------------------------------------------------------------
// Send Record
//------------------------------------------------------------------------------
// There are 3 different record conditions to be created/updated:
//   1:  New record,  IEN = '+1,', CommentsOnly := FALSE
//       This is a new record 
//   2:  Update record, IEN = SelectedIEN, CommentsOnly := FALSE; 
//       This is an existing record updated before a consult is assigned
//   3:  Update record, IEN = SelectedIEN, CommentsOnly := TRUE;
//       Once a consult has been assigned, the only field that can be 
//       updated is the Comments/Notes field
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.SendRecord(RecordIEN : string; CommentsOnly : Boolean);
var
  FMDateToday : string;
  buffer : string;
begin
    frmMain.FMValidator1.Clear;
    FMDateToday := FMToDateConvert(FormatDateTime('mm/dd/yyyy',now));
    buffer := FormatDateTime('hhnn',now);
    FMDateToday := FMDateToday + '.' + buffer;
    FMFilerVR.Clear;
    if CommentsOnly then // only updating comments, all other fields are read-only at this point
    begin
      FMCommentsNotes.IENS := RecordIEN;
      FMFilerVR.AddChgdControl(FMCommentsNotes); // Comments / Notes
    end
    else // updating all fields
    begin
      if RecordIEN = '+1,' then  // Only include date/time on a new record
      begin
        frmMain.FMValidator1.SetUp('396.9',RecordIEN,'.01',FmDateToday);
        If frmMain.FMValidator1.Validate then
          FMFilerVR.AddFDA('396.9',RecordIEN,'.01',frmMain.FMValidator1.Results[0])  // New record IEN - include request date/time
        else
          frmMain.FMValidator1.DisplayErrors;
      end;
      FMFilerVR.AddFDA('396.9',RecordIEN,'3',FMRegionalOfficeNumberIEN.Text);  // Regional Office
      FMFilerVR.AddFDA('396.9',RecordIEN,'4',PatientIEN);  // Patient
      FMFilerVR.AddFDA('396.9',RecordIEN,'5',ConvertRichEditToString(reRehabObjectives));  // Rehab Objective of Veteran
      FMFilerVR.AddFDA('396.9',RecordIEN,'6',edtReasonForReferral.Text);  // Reason for Referral
//      FMFilerVR.AddFDA('396.9',RecordIEN,'7',FMToDateConvert(FormatDateTime('mm/dd/yyyy',dtRehabilitation.Date)));  // Anticipated Date of Rehabilitation
      FMFilerVR.AddFDA('396.9',RecordIEN,'7',FMToDateConvert(leAnticipatedDate.Text));  // Anticipated Date of Rehabilitation
      FMFilerVR.AddFDA('396.9',RecordIEN,'8',IntToStr(Integer(cbMedicalServicesRequested.Items.Objects[cbMedicalServicesRequested.ItemIndex])));  // Medical Services Requested
      FMFilerVR.AddFDA('396.9',RecordIEN,'8.1',edtServicesOther.Text);  // Other Reason
//    if dtPreferredScheduleDate.Format = ShortDateFormat then // date exists
      if lePreferredDate.Text <> '' then // date exists       
      begin
//      FMFilerVR.AddFDA('396.9',RecordIEN,'9',FMToDateConvert(FormatDateTime('mm/dd/yyyy',dtPreferredScheduleDate.Date)));  // Preferred Schedule Date
        FMFilerVR.AddFDA('396.9',RecordIEN,'9',FMToDateConvert(lePreferredDate.Text));  // Preferred Schedule Date
        FMFilerVR.AddFDA('396.9',RecordIEN,'10',edtPreferredScheduleDateReason.Text);  // Preferred Schedule Date Reason
      end;
      FMFilerVR.AddFDA('396.9',RecordIEN,'11',frmMain.VocRehab.GetPointOfContactIEN(TRUE));  // Point of Contact
      FMCommentsNotes.IENS := RecordIEN;
      FMFilerVR.AddChgdControl(FMCommentsNotes); // Comments / Notes
      FMFilerVR.AddFDA('396.9',RecordIEN,'13','N');  // Request Status
    end;
    If Not FMFilerVR.Update Then
    Begin
      FMFilerVR.DisplayErrors;
      exit;
    End
    else
    begin
      // if adding a new record, we need to populate SelectMedicalRequestIEN 
      // with the recently recreated IEN prior to sending the Mail message
      if RecordIEN = '+1,' then 
         SelectedMedicalRequestIEN := FMFilerVR.FindIen('+1,');  // assign new IEN to SelectedMedicalRequestIEN
    end;
end;

//------------------------------------------------------------------------------
// Convert RichEdit Text field to a single plain text string
//------------------------------------------------------------------------------
function TfrmVRMedicalRequest.ConvertRichEditToString(reInput: TRichEdit): string;
var
  i : Integer;
  buffer : string;
begin
  buffer := '';
  for i := 0 to reInput.Lines.Count - 1 do
     buffer := buffer + reInput.Lines[i] + ' ';
  Result := buffer;    
end;

//------------------------------------------------------------------------------
// Set Edits To Read Only
//------------------------------------------------------------------------------
// If a consult has been assigned, the only editable control will be the
// Comments/Notes field.  Pressing Submit now only updates that one field.
// The POC button is enabled for viewing in any mode.
//
// The Assign Consult button is only available to those who have the
// VHA_ACCESS security key.  Even if a consult has been assigned, if the user
// has that key, the Assign Consult button will be enabled.
//
// The main purpose of the read only edit fields is to comply with VA508
// compatibility.  Read only controls are better than invisible/disabled
// controls.
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.SetEditsToReadOnly(Value : Boolean);
begin

  If Value then  // Controls should be read only
  begin
    // Anticipated Date of Rehabilitation
//    lblAnticipatedDateOfRehab.FocusControl := edtRehabDateReadOnly;
//    edtRehabDateReadOnly.Top := leAnticipatedDate.Top;
//    dtRehabilitation.Visible := FALSE;
//    leAnticipatedDate.Visible := FALSE;
    leAnticipatedDate.ReadOnly := TRUE;
    leAnticipatedDate.Color := clBtnFace;
    btnCalAnticipatedDate.Visible := FALSE;
    // Regional Office
    lblRegionalOffice.FocusControl := edtRegionalOfficeReadOnly;
    edtRegionalOfficeReadOnly.Top := FMRegionalOfficeNumber.Top;
    FMRegionalOfficeNumber.Visible := FALSE;
    edtRegionalOfficeReadOnly.Visible := TRUE;
    if VAUtils.ScreenReaderActive then
      edtRegionalOfficeReadOnly.Enabled := TRUE;
    // Medical Services Requested
    lblMedicalServicesRequested.FocusControl := edtMedicalServicesRequestedReadOnly;
    edtMedicalServicesRequestedReadOnly.Top := cbMedicalServicesRequested.Top;
    cbMedicalServicesRequested.Visible := FALSE;
    edtMedicalServicesRequestedReadOnly.Visible := TRUE;
    // Other Reason
    edtServicesOther.ReadOnly := TRUE;
    edtServicesOther.Color := clBtnFace;
    // Reason for Referral
    edtReasonForReferral.ReadOnly := TRUE;
    edtReasonForReferral.Color := clBtnFace;
    // Rehabilitation Objectives
    reRehabObjectives.ReadOnly := TRUE;
    reRehabObjectives.Color := clBtnFace;
    // Preferred Schedule Date
//    lblPreferredScheduleDate.FocusControl := edtPreferredScheduleDateReadOnly;
//    edtPreferredScheduleDateReadOnly.Top := lePreferredDate.Top;
//    edtPreferredScheduleDateReadOnly.ReadOnly := TRUE;
//    edtPreferredScheduleDateReadOnly.Visible := TRUE;
//    dtPreferredScheduleDate.Visible := FALSE;
    lePreferredDate.ReadOnly := TRUE;
    lePreferredDate.Color := clBtnFace;
    btnCalPreferredDate.Visible := FALSE;
    // Preferred Schedule Date Reason
    lblPreferredScheduleDateReason.Font.Color := clBlack;
    edtPreferredScheduleDateReason.Color := clBtnFace;
    edtPreferredScheduleDateReason.ReadOnly := TRUE;
    // Consult Date
    edtSelectedConsult.ReadOnly := TRUE;
    edtSelectedConsult.Color := clBtnFace;
    // Send Medical Services Button
    btnSendRequest.Visible := FALSE;
    // Cancel Medical Services Button
    btnCancelRequest.Visible := FALSE;
  end
  else  // Controls are editable
  begin
    // Anticipated Date of Rehabilitation
//    lblAnticipatedDateOfRehab.FocusControl := dtRehabilitation;
//    dtRehabilitation.Visible := TRUE;
//    edtRehabDateReadOnly.Visible := FALSE;
    leAnticipatedDate.ReadOnly := FALSE;
    leAnticipatedDate.Color := clWindow;
    btnCalAnticipatedDate.Visible := TRUE;
    // Regional Office
    lblRegionalOffice.FocusControl := FMRegionalOfficeNumber;
    if OneRegionalOffice then
    begin
      FMRegionalOfficeNumber.Visible := FALSE;
      edtRegionalOfficeReadOnly.Visible := TRUE;
      if VAUtils.ScreenReaderActive then
        edtRegionalOfficeReadOnly.Enabled := TRUE;
    end
    else
    begin
      FMRegionalOfficeNumber.Visible := TRUE;
      edtRegionalOfficeReadOnly.Visible := FALSE;
    end;
    // Medical Services Requested
    lblMedicalServicesRequested.FocusControl := cbMedicalServicesRequested;
    cbMedicalServicesRequested.Visible := TRUE;
    edtMedicalServicesRequestedReadOnly.Visible := FALSE;
    // Reason for Referral
    edtReasonForReferral.ReadOnly := FALSE;
    edtReasonForReferral.Color := clWindow;
    // Rehabilitation Objectives
    reRehabObjectives.ReadOnly := FALSE;
    reRehabObjectives.Color := clWindow;
    // Preferred Schedule Date
//    lblPreferredScheduleDate.FocusControl := dtPreferredScheduleDate;
//    edtPreferredScheduleDateReadOnly.Visible := FALSE;
//    dtPreferredScheduleDate.Visible := TRUE;
    lePreferredDate.ReadOnly := FALSE;
    lePreferredDate.Color := clWindow;
    btnCalPreferredDate.Visible := TRUE;
    if lePreferredDate.Text = '' then // date exists so enable reason for editing
      edtPreferredScheduleDateReason.ReadOnly := TRUE
    else
      edtPreferredScheduleDateReason.ReadOnly := FALSE;
    // Consult Date
//  edtSelectedConsult.ReadOnly := FALSE;
    edtSelectedConsult.Color := clWindow;
    // Send Medical Services Button
    btnSendRequest.Visible := TRUE;
    // Cancel Medical Services Button
    btnCancelRequest.Visible := TRUE;
  end;
  // Comments are enabled while the request is new or pending
  // Comments are disabled when the request is cancelled or completed
  if (frmMain.VocRehab.RequestCancelled) OR
     (frmMain.VocRehab.RequestCompleted) then
  begin
     if VAUtils.ScreenReaderActive then
     begin
       pnlComments.Enabled := TRUE;      // Allow the field to be read by the
       FMCommentsNotes.ReadOnly := TRUE; // screen reader even if disabled
     end
     else
     begin
       pnlComments.Enabled := FALSE;     // don't even tab to that field if the
     end;                                // screen reader is not active
     FMCommentsNotes.Color := clBtnFace;
     btnSendRequest.Visible := FALSE;
  end
  else // Request is not cancelled/complete so enable comments field
  begin
     pnlComments.Enabled := TRUE;
     FMCommentsNotes.ReadOnly := FALSE;
     FMCommentsNotes.Color := clWindow;
     btnSendRequest.Visible := TRUE;
     btnAssignConsults.Enabled := EnableAssignConsultsButton;
  end;
end;

//------------------------------------------------------------------------------
// Setup New Exam
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.SetupNewExam;
begin
  ClearAndInitiateFields;
  frmMain.VocRehab.RequestCancelled := FALSE;
  frmMain.VocRehab.RequestCompleted := FALSE;
  pnlConsultsSelected.Visible := FALSE;
  btnCancelRequest.Visible := ShowCancelRequestButton;
  edtSelectedConsult.ReadOnly := TRUE;
  edtSelectedConsult.Color := clBtnFace;
  lblConsultsSelected.Font.Color := clGray;
  btnSendRequest.Visible := TRUE;
end;

//------------------------------------------------------------------------------
// Setup Existing Exam
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.SetupExistingExam;
begin
  ClearAndInitiateFields;
  pnlConsultsSelected.Visible := TRUE;
  PopulateFieldValues;
  btnCancelRequest.Visible := ShowCancelRequestButton;
  btnAssignConsults.Enabled := EnableAssignConsultsButton;
  btnSendRequest.Caption := 'Update Medical Services Request';
end;

//------------------------------------------------------------------------------
// Populate Field Values
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.PopulateFieldValues;
var
  buffer : string;
begin
  FMGetsVR.FileNumber := '396.9';
  FMGetsVR.FieldNumbers.Add('.01');  // Request Date
  FMGetsVR.FieldNumbers.Add('1');    // Send To Routing Location
  FMGetsVR.FieldNumbers.Add('3');    // Regional Office
  FMGetsVR.FieldNumbers.Add('5');    // Rehabilitation Objectives
  FMGetsVR.FieldNumbers.Add('6');    // Reason for Referral
  FMGetsVR.FieldNumbers.Add('7');    // Anticipated Date of Rehabilitation
  FMGetsVR.FieldNumbers.Add('8');    // Medical Services Requested
  FMGetsVR.FieldNumbers.Add('8.1');  // Other Reason
  FMGetsVR.FieldNumbers.Add('9');    // Preferred Schedule Date
  FMGetsVR.FieldNumbers.Add('10');   // Preferred Schedule Date Reason
  FMGetsVR.FieldNumbers.Add('12');   // Comments / Notes
  FMGetsVR.FieldNumbers.Add('14');   // Consults
  FMGetsVR.IENS := SelectedMedicalRequestIEN;      // Populated on main form / on click event
  FMGetsVR.GetData;
  if FMGetsVR.Results.Count < 1 then
     // Show Error
  else
  begin
     try
       edtDateSubmitted.Text := 'Request Date: ' + FMGetsVR.GetField('.01').FMDBExternal;
       edtRegionalOfficeReadOnly.Text := FMGetsVR.GetField('3').FMDBExternal; // read only display
       FMRegionalOfficeNumberIEN.Text := FMGetsVR.GetField('3').FMDBInternal;
       FMRegionalOfficeNumber.Visible := FALSE;
       reRehabObjectives.Lines.Add(FMGetsVR.GetField('5').FMDBExternal);
       edtReasonForReferral.Text := FMGetsVR.GetField('6').FMDBExternal;
//       buffer := ConvertCalendarDateToDelphiDate(FMGetsVR.GetField('7').FMDBExternal);
//       dtRehabilitation.Format := ShortDateFormat;
//       dtRehabilitation.Date := StrToDate(buffer);
//       edtRehabDateReadOnly.Text := DateToStr(dtRehabilitation.Date);  // read only display
       leAnticipatedDate.Text := ConvertCalendarDateToDelphiDate(FMGetsVR.GetField('7').FMDBExternal);
       leAnticipatedDate.ReadOnly := TRUE;  // read only display
       buffer := FMGetsVR.GetField('8').FMDBExternal;
       cbMedicalServicesRequested.ItemIndex := cbMedicalServicesRequested.Items.IndexOf(buffer);
       edtMedicalServicesRequestedReadOnly.Text := cbMedicalServicesRequested.Items[cbMedicalServicesRequested.ItemIndex];
       edtServicesOther.Text := FMGetsVR.GetField('8.1').FMDBExternal;
       if edtServicesOther.Text <> '' then
       begin
         lblServicesOther.Font.Color := clBlack;
         edtServicesOther.Color := clWhite;
         edtServicesOther.Font.Color := clBlack;
       end
       else
       begin
         lblServicesOther.Font.Color := clGray;
         edtServicesOther.Color := clBtnFace;
         edtServicesOther.Clear;
       end;
       buffer := ConvertCalendarDateToDelphiDate(FMGetsVR.GetField('9').FMDBExternal);
       if (buffer = '') or (buffer = '//') then // if blank date string
          begin
//          dtPreferredScheduleDate.Format := ' '; // display blank date
//          dtPreferredScheduleDate.Date := Now; // set to today in case the control is opened
          lePreferredDate.Text := ''; // display blank date
          lblPreferredScheduleDateReason.Font.Color := clGray;
          edtPreferredScheduleDateReason.Color := clBtnFace;
          edtPreferredScheduleDateReason.Clear;
          end
       else // valid date - get reason field too
       begin
//          dtPreferredScheduleDate.Format := ShortDateFormat;
//          dtPreferredScheduleDate.Date := StrToDate(buffer);
//          edtPreferredScheduleDateReadOnly.Text := DateToStr(dtPreferredScheduleDate.Date);
          lePreferredDate.Text := buffer;
          edtPreferredScheduleDateReason.Text := FMGetsVR.GetField('10').FMDBExternal;
          if edtPreferredScheduleDateReason.Text <> '' then
          begin
            lblPreferredScheduleDateReason.Font.Color := clBlack;
            edtPreferredScheduleDateReason.Color := clWhite;
            edtPreferredScheduleDateReason.Font.Color := clBlack;
          end
          else
          begin
            lblPreferredScheduleDateReason.Font.Color := clGray;
            edtPreferredScheduleDateReason.Color := clBtnFace;
            edtPreferredScheduleDateReason.Clear;
          end;
       end;
       FMCommentsNotes.Lines.AddStrings(FMGetsVR.GetField('12').FMWPTextLines);
       pnlConsultsSelected.Visible := TRUE;
       PopulateConsultSelectedField;  // Get and populate consult assigned/linked field
     except
       ShowMessageCAPRI('Error populating fields');
     end;
  end;
end;

//------------------------------------------------------------------------------
// Populate Field Values
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.PopulateConsultSelectedField;
var
  aIEN : string;
begin     
     // populate Consult Selected field if available
     edtSelectedConsult.Clear;
     FMListerVocRehab.Screen := '';
     FMListerVocRehab.FMIndex := '';
     FMListerVocRehab.DisplayFields.Clear;
     FMListerVocRehab.FieldNumbers.Clear;
     FMListerVocRehab.IENS := ',' + SelectedMedicalRequestIEN;
     FMListerVocRehab.FileNumber := '396.914';
     FMListerVocRehab.DisplayFields.Add('.01');
     FMListerVocRehab.FieldNumbers.Add('.01');
     FMListerVocRehab.GetList(nil);
     if FMListerVocRehab.Results.Count < 1 then
     begin
       edtSelectedConsult.Text := NO_CONSULTS_FOUND;
       Exit;
     end
     else
     begin  
       // while this field is a multiple field, current restrictions have a 
       // "one consult to one form 8861" so there should only be one entry in this list
       aIEN := FMListerVocRehab.GetRecord(FMListerVocRehab.Results[0]).GetField('.01').FMDBInternal; // get the first entry only
     end;
     FMGetsVR.FileNumber := '123';
     FMGetsVR.FieldNumbers.Add('.01');  // Request Date
     FMGetsVR.FieldNumbers.Add('1');    // Send To Location (Regional Office)
     FMGetsVR.IENS := aIEN;
     FMGetsVR.GetData;
     if FMGetsVR.Results.Count < 1 then
        // no records
     else
     begin
        edtSelectedConsult.Text := FMGetsVR.GetField('.01').FMDBExternal + ' - ' +
                                   FMGetsVR.GetField('1').FMDBExternal;
     end;
end;


//------------------------------------------------------------------------------
// Are Required Fields Complete
//------------------------------------------------------------------------------
// Requirement Fields:  
//   * Anticipated Date of Rehabilitation cannot be blank 
//   * If Reason for Referal is OTHER, then OTHER description must not be blank
//   * Reason for Referal can't be blank
//   * if Preferred Schedule date isn't blank, then Preferred Schedule Date
//     reason can't be blank
//   * if screen reader is active and preferred schedule date or anticipated
//     date of rehabilitation is today, then the field isn't correct
//   * Reason for Referral - required field
//   * Rehabilitation Objectives - required field
//------------------------------------------------------------------------------
function TfrmVRMedicalRequest.AreRequiredFieldsComplete: Boolean;
begin
  Result := TRUE;
  // If the datetime control is not visible, we are in read-only mode.
  // Don't do field validation in read-only mode.
//   if dtRehabilitation.Visible = FALSE then
  if leAnticipatedDate.ReadOnly = TRUE then
     exit;

  if frmMain.VocRehab.AddExam then // don't validate date when "editing" as a past date may be valid then
  begin
//    if (dtRehabilitation.Format <> ShortDateFormat) then
    if leAnticipatedDate.Text = '' then
    begin
      Result := FALSE;
      ShowMessageCAPRI('Anticipated date of rehabilitation cannot be blank.');
      exit;
    end;
//    if (dtRehabilitation.Date < Tomorrow) then
//    begin
//      Result := FALSE;
//      ShowMessageCAPRI('Anticipated date of rehabilitation must be a future date.');
//      exit;
//    end;
  end;    
  if FMRegionalOfficeNumberIEN.Text = '' then
  begin
     ShowMessageCAPRI('Regional Office must be completed.');
     Result := FALSE;
     exit;
  end;
  if cbMedicalServicesRequested.ItemIndex = -1 then
  begin
     ShowMessageCAPRI('Medical Services Requested field cannot be blank.');
     Result := FALSE;
     exit;
  end;
  if (UpperCase(cbMedicalServicesRequested.Items[cbMedicalServicesRequested.ItemIndex]) = REASON_OTHER) AND
     (Length(edtServicesOther.Text)< 3) then
  begin
     ShowMessageCAPRI('Other Description field must be completed.');
     Result := FALSE;
     exit;
  end;
//  if (dtPreferredScheduleDate.Format = ShortDateFormat) AND 
//     (length(edtPreferredScheduleDateReason.Text) < 3) then
  if (lePreferredDate.Text <> '') AND 
     (length(edtPreferredScheduleDateReason.Text) < 3) then
  begin
     ShowMessageCAPRI('Setting a Preferred Schedule Date must include a reason.');
     Result := FALSE;
     exit;
  end;
  if Length(edtReasonForReferral.Text) < 3 then
  begin
     ShowMessageCAPRI('Reason for Referral field must be completed.');
     Result := FALSE;
     exit;
  end;
  if Length(reRehabObjectives.Text) < 3 then
  begin
     ShowMessageCAPRI('Rehabilitation Objectives must be completed.');
     Result := FALSE;
     exit;
  end;
end;


//------------------------------------------------------------------------------ 
// VA508 Compatability - don't disable, just gray out and make read only
//------------------------------------------------------------------------------ 
procedure TfrmVRMedicalRequest.cbMedicalServicesRequestedChange(Sender: TObject);
begin
   if UpperCase(cbMedicalServicesRequested.Items[cbMedicalServicesRequested.ItemIndex]) = REASON_OTHER then
   begin
      lblServicesOther.Font.Color := clBlack;
      edtServicesOther.ReadOnly := FALSE;
      edtServicesOther.Clear;
      edtServicesOther.Font.Color := clBlack;
      edtServicesOther.Color := clWhite;
      if frmVRMedicalRequest.Visible then
         edtServicesOther.SetFocus;
   end
   else
   begin
      lblServicesOther.Font.Color := clGray;
      edtServicesOther.Clear;
      edtServicesOther.Color := clBtnFace;
      edtServicesOther.ReadOnly := TRUE;
   end;
end;

//------------------------------------------------------------------------------
// Clear And Initiate Fields
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.ClearAndInitiateFields;
begin
   edtVAClaimNumber.Text := 'VA Claim #: ' + frmMain.VocRehab.VAClaimNumber;
   edtPatientName.Text := 'Patient Name: ' + PatientName;
   edtDateSubmitted.Text := 'Request Date:    ' + DateToStr(now);
   // Regional Office
   GetRegionalOfficeValues;
   // Medical Services Requested
   GetMedicalServicesRequestedValues;
   // Preferred Schedule Date
//   dtPreferredScheduleDate.Date := now;
   // Anticipated Date of Rehabilitation
//   dtRehabilitation.Date := now;
   // Date fields
//   dtPreferredScheduleDate.Format := ' ';  // screen reader requires Alt + Down arrow to select month
//   dtRehabilitation.Format := ' ';  
   // Rehabilitation Objectives
   reRehabObjectives.Lines.Clear;
   // Reason For Referral
   edtReasonForReferral.Text := '';
   edtServicesOther.Text := '';
   edtServicesOther.ReadOnly := TRUE;
   edtServicesOther.Font.Color := clGrayText;
   edtServicesOther.Color := clBtnFace;
   // Preferred Schedule Date
//   dtPreferredScheduleDate.Date := Now;
   edtPreferredScheduleDateReason.Clear;
   edtPreferredScheduleDateReason.Color := clBtnFace;
   edtPreferredScheduleDateReason.Font.Color := clGrayText;
   edtPreferredScheduleDateReason.ReadOnly := TRUE;
   lblPreferredScheduleDateReason.Font.Color := clGray;
   // Comments or Notes
   FMCommentsNotes.Lines.Clear;
   // Consults (only if edit has been pressed previously)
   edtSelectedConsult.Clear;
   
end;


//------------------------------------------------------------------------------
// Date: Preferred Schedule Date Change
//------------------------------------------------------------------------------
// Property inspector for TDateTimePicker has a format := ' ' to show a blank 
// date upon startup.  Upon change, the format property should be changed to 
// the ShortDateFormat;
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.dtPreferredScheduleDateChange(Sender: TObject);
begin
  (Sender as TDateTimePicker).Format:=ShortDateFormat;  // this is needed to support a "blank" optional date
  // VA508 Compatability - don't disable, just gray out and make read only
  lblPreferredScheduleDateReason.Font.Color := clBlack;
  edtPreferredScheduleDateReason.ReadOnly := FALSE;
  edtPreferredScheduleDateReason.Color := clWhite;
  edtPreferredScheduleDateReason.Font.Color := clBlack;
  if frmVRMedicalRequest.Visible then
     edtPreferredScheduleDateReason.SetFocus;
end;

//------------------------------------------------------------------------------
// Date: Anticipated Date of Rehabilitation Change
//------------------------------------------------------------------------------
// Property inspector for TDateTimePicker has a format := ' ' to show a blank 
// date upon startup.  Upon change, the format property should be changed to 
// the ShortDateFormat;
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.dtRehabilitationChange(Sender: TObject);
begin
  (Sender as TDateTimePicker).Format:=ShortDateFormat;  // this is needed to support a "blank" optional date
end;


//------------------------------------------------------------------------------
// Enable Assign Consults Button
//------------------------------------------------------------------------------
// Assign Consults button is never visible on the Add Exam screen.  On the
// Edit Exam screen, it is only visible if the user has VHA Coordinator login
// credentials AND if a consult is not already assigned.  If an exam is
// cancelled or completed, it should not be enabled.
// Otherwise, the button stays hidden.
//------------------------------------------------------------------------------
function TfrmVRMedicalRequest.EnableAssignConsultsButton: Boolean;
begin
  Result := FALSE;
  if frmMain.VocRehab.AddExam then 
     Exit;
  if frmMain.VocRehab.VHAAccess = FALSE then
     Exit;
  if (frmMain.VocRehab.RequestCancelled) OR
     (frmMain.VocRehab.RequestCompleted) then
     Exit;
  if edtSelectedConsult.Text = NO_CONSULTS_FOUND then
     Result := TRUE
  else
     Result := FALSE;
end;

//------------------------------------------------------------------------------
// Show Cancel Request Button
//------------------------------------------------------------------------------
// The Cancel Request button is not visible when creating a new exam request.
// The Cancel Request button is only available when the Consult status is NEW.
// This means that no consults have been assigned to the request.  If no 
// consults are assigned, then allow the user to cancel the request.
//------------------------------------------------------------------------------
function TfrmVRMedicalRequest.ShowCancelRequestButton: Boolean;
begin
  Result := FALSE;  // don't show button
  if frmMain.VocRehab.AddExam then 
     Exit;
  if (edtSelectedConsult.Text = '') OR (edtSelectedConsult.Text = NO_CONSULTS_FOUND) then // consult not assigned yet
     Result := TRUE;  // show button
end;

//------------------------------------------------------------------------------
// Button - Anticipated Date Calendar Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnCalAnticipatedDateClick(Sender: TObject);
var
  ADate : TDateTime;
begin
  frmVRSelectDate := TfrmVRSelectDate.Create(Application);
  try
    if frmMain.VocRehab.AddExam = TRUE then  // new exam - enforce no previous dates
       frmVRSelectDate.MinimumDate := Tomorrow
    else
       frmVRSelectDate.MinimumDate := StrToDate('01/01/1900');  // editing - allow reasonable dates
    if TryStrToDate(leAnticipatedDate.Text,ADate) then
      frmVRSelectDate.calSelectDate.Date := ADate
    else
      frmVRSelectDate.calSelectDate.Date := Tomorrow;
    frmVRSelectDate.ShowModal;
    leAnticipatedDate.Text := frmVRSelectDate.SelectedDate;
  finally
    FreeAndNil(frmVRSelectDate);
  end;
  if (ActiveControl = leAnticipatedDate) then
    SelectNext(ActiveControl as TWinControl, TRUE, TRUE);
end;

//------------------------------------------------------------------------------
//  Button - Preferred Date Calendar Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnCalPreferredDateClick(Sender: TObject);
var
  ADate : TDateTime;
begin     
  frmVRSelectDate := TfrmVRSelectDate.Create(Application);
  try
    if frmMain.VocRehab.AddExam = TRUE then  // new exam - enforce no previous dates
       frmVRSelectDate.MinimumDate := Tomorrow
    else
       frmVRSelectDate.MinimumDate := StrToDate('01/01/1900');  // editing - allow reasonable dates
    if TryStrToDate(lePreferredDate.Text,ADate) then
      frmVRSelectDate.calSelectDate.Date := ADate
    else
      frmVRSelectDate.calSelectDate.Date := Tomorrow;
    frmVRSelectDate.ShowModal;
    lePreferredDate.Text := frmVRSelectDate.SelectedDate;
  finally
    FreeAndNil(frmVRSelectDate);
  end;
  if lePreferredDate.Text <> '' then
  begin
    edtPreferredScheduleDateReason.ReadOnly := FALSE;
    lblPreferredScheduleDateReason.Font.Color := clBlack;
    edtPreferredScheduleDateReason.Color := clWhite;
    edtPreferredScheduleDateReason.Font.Color := clBlack;
    edtPreferredScheduleDateReason.SetFocus;
  end
  else
  begin
    edtPreferredScheduleDateReason.ReadOnly := TRUE;
    lblPreferredScheduleDateReason.Font.Color := clGray;
    edtPreferredScheduleDateReason.Color := clBtnFace;
  end;
end;

//------------------------------------------------------------------------------
// Date Complete - Date validation function
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.DateComplete(var StrLikeDate : string; MinDate : TDate; const NowIfError : Boolean = TRUE);
var
   DateStr : string;
   Year, Month, Day : Word;
   cnt, SepCount : Integer;
begin
   DateStr := StrLikeDate;
   if DateStr = '' then
      Exit;
   SepCount := 0;
   for cnt := 1 to Length(DateStr) do
   begin
     if not (DateStr[cnt] in ['0'..'9']) then
     begin
       DateStr[cnt] := DateSeparator;
       inc(SepCount);
     end;
   end;
   while (DateStr <> '') and (DateStr[Length(DateStr)]=DateSeparator) do
   begin
     Delete(DateStr, Length(DateStr),1);
     Dec(SepCount);
   end;
   DecodeDate(Now,Year,Month,Day);
   if SepCount = 0 then
   begin
     case Length(DateStr) of
       0   : DateStr := DateToStr(Now);
       1, 2: DateStr := DateStr+DateSeparator+IntToStr(Day);
//     1, 2: DateStr := DateStr+DateSeparator+IntToStr(Month);  // use this if short date format is dd/mm/yyyy
       4   : Insert(DateSeparator, DateStr, 3);
       6, 8: begin
             Insert(DateSeparator, DateStr, 5);
             Insert(DateSeparator, DateStr, 3);
             end; 
     end; // case
   end; // if SepCount
   try
     StrLikeDate := FormatDateTime('mm/dd/yyyy',StrToDate(DateStr));
   Except
     if NowIfError = TRUE then
     begin
      StrLikeDate := DateToStr(Date)
     end
     else
        StrLikeDate := '';
   end;
   // Date is valid, check for greater than minimum date
   if StrToDate(DateStr) < MinDate then
      StrLikeDate := 'Min ' + StrLikeDate;
end;


end.
