unit dlgVndrExmChkInRslts;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Fmcntrls, untMiscMthds, clsPatientInfo, ExtCtrls,
  { VistA Components }
  MFunStr,
  { SFTP Components }
  clsSFTPConn,
  { PDF Engine Components }
  SQLite3Pages;

type
  TdlgVndrExamRslts = class(TForm)
    fmMemoExmRslts: TFMMemo;
    Panel1: TPanel;
    Label2: TLabel;
    btnOk: TButton;
    btnSaveDflt: TButton;
    tmpMemo: TMemo;
    btnPrint: TButton;
    rchEdtTmpVndrRslts: TRichEdit;
    procedure btnPrintClick(Sender: TObject);
  private
    { Private declarations }
    _VndrExmFiles: TStringList;
    _SFTPClient: TSFTPConn;
    procedure GetExmRslts;
    procedure PrintExamResults;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent; PtntInfo: TPatientInfo; VndrExmFles:TStrings;
         SFTPConn: TSFTPConn; TmrVal: Cardinal); reintroduce;
    destructor Destroy; override;
  end;

var
  dlgExamCmnts: TdlgVndrExamRslts;

implementation

{$R *.dfm}

procedure TdlgVndrExamRslts.btnPrintClick(Sender: TObject);
begin
  PrintExamResults;
end;

constructor TdlgVndrExamRslts.Create(AOwner: TComponent; PtntInfo: TPatientInfo;
     VndrExmFles:TStrings; SFTPConn: TSFTPConn; TmrVal: Cardinal);
begin
  inherited Create(AOwner);
  _VndrExmFiles := TStringList.Create;
  Caption := '2507 Vendor Exam Result(s) for: ' + PtntInfo.PatientName + ' (' +
       Copy(PtntInfo.PatientSSN, Length(PtntInfo.PatientSSN)-3, 4) + ')';
  _SFTPClient := SFTPConn;
  if (Not (VndrExmFles = nil)) then
    _VndrExmFiles.Assign(VndrExmFles);
  GetExmRslts;  { Load Exam Results into Memo Area for Review }
end;

destructor TdlgVndrExamRslts.Destroy;
var
  i: Integer;
begin
  { Delete temporary downloaded Vendor files from User system }
  for i := 0 to _VndrExmFiles.Count - 1 do
    if FileExists(Piece(_VndrExmFiles[i], '^', 2)) then
      try
        DeleteFile(Piece(_VndrExmFiles[i], '^', 2));
      except { Do Nothing on Errors }
      end;  { End Try/Except Block }
  _VndrExmFiles.Free;
  inherited;
end;

{ Private }

procedure TdlgVndrExamRslts.GetExmRslts;
var
  i: Integer;
begin
  fmMemoExmRslts.Lines.Clear;
  { Download Files from Vendor SFTP to Temporary Directory }
  _SFTPClient.DownloadFiles(_VndrExmFiles);
  { Merge, if applicable, and display completed vendor exam results in the Memo }
  for i := 0 to _VndrExmFiles.Count - 1 do
  begin
    tmpMemo.Lines.LoadFromFile(Piece(_VndrExmFiles[i], '^', 2));
    fmMemoExmRslts.Lines.AddStrings(tmpMemo.Lines);
    if (i <> (_VndrExmFiles.Count - 1)) then
      fmMemoExmRslts.Lines.Add(fmMemoExmRslts.Lines.LineBreak);
  end;  { End For Loop through Selected Vendor Exam Files }
end;

procedure TdlgVndrExamRslts.PrintExamResults;
var
  rptObj: TGDIPages;
begin
  { Create GDI to print from not directly from Memo }
  rptObj := TGDIPages.Create(self);
  try
    with RptObj do
    begin
      BeginDoc;  { Open report document for editing }
      PageMargins := Rect(10,10,10,10);
      Font.Name := 'Courier New';  { Monospaced font }
      Font.Size := 12;
      NewHalfLine;
      WordWrapLeftCols := True;
      { No Header or Footers just Text Body which mirrors Memo Info -
        If Header/Footer needed in future see frm508ReportRslts for example. }
      QuickCopy(fmMemoExmRslts.Lines, rchEdtTmpVndrRslts);
      RptObj.AppendRichEdit(rchEdtTmpVndrRslts.Handle);
      EndDoc;  { Close report document }
      try
        //ShowPreviewForm;  { For Troubleshooting to view contents - PrintPreview }
        PrintPages(-1, -1);  { Print from GDI Object NOT Memo Display }
      except
        on E: Exception do
        begin
          Application.MessageBox(PChar(E.Message), 'Printer Error', MB_OK);
        end;  { End Exception Handling - IE. No Printer Driver Found }
      end;  { End Inner Try/Except Block }
    end;  { End With Report Object }
  finally
    rptObj.Free;
  end;  { End Outer Try/Finally Block }
end;

end.
