unit frmVVA;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, Grids, ImgList, ExtCtrls, CheckLst, TypInfo,
  virtualVAWebServiceClient, VA508ImageListLabeler, VA508AccessibilityManager;

type


  TfrmVirtualVA = class(TForm)
    btnClose: TButton;
    imgListStatusIcons: TImageList;
    Panel1: TPanel;
    imgFormatText: TImage;
    edFormatText: TEdit;
    imgTransmitFile: TImage;
    edTransmitFile: TEdit;
    imgDeleteFile: TImage;
    edDeleteFile: TEdit;
    lblVVAResult: TLabel;
    mmoVVAResult: TMemo;
    edVirtualVATitle: TEdit;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    VA508ImageListLabeler1: TVA508ImageListLabeler;
    procedure btnCloseClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
    FSendToVVADone : Boolean;
    procedure OnUpdateStatus(Sender: TObject; Action: TvvaAction; Status: TvvaStatus);
    procedure UpdateStatusDisplay(anEditBox: TEdit; aStatusImg: Timage; aStatus: TvvaStatus);
  public
    { Public declarations }
    vvaWSC: TvvaWSC;
    constructor Create(AOwner: TComponent); Override;
    destructor Destroy; Override;
  end;

var
  frmVirtualVA: TfrmVirtualVA;

implementation
{$R *.dfm}

{ TfrmVirtualVA }

procedure TfrmVirtualVA.btnCloseClick(Sender: TObject);
begin
  Close;
end;


constructor TfrmVirtualVA.Create(AOwner: TComponent);
begin
  inherited;
  vvaWSC := TvvaWSC.Create;
  vvaWSC.OnUpdateStatus := OnUpdateStatus;
  FSendToVVADone := False;
end;

destructor TfrmVirtualVA.Destroy;
begin
  FreeAndNil(vvaWSC);
  inherited;
end;

procedure TfrmVirtualVA.FormActivate(Sender: TObject);
begin
  {Since the modal form receives messages from the
   vvaWSC object, we need to prevent any handlers from
   from triggering SendReportToVVA more than once}
  if (not FSendToVVADone) and (not vvaWSC.IsRunning) then
  begin
    mmoVVAResult.Text := vvaWSC.SendFileToVVA;
    FSendToVVADone := True;
  end;
end;

{=========================================================================
 OnUpdateStatus
 This event handler updates the Virtual VA status dialog box with the
 progress of the local document creation, transmission, and deletion.
 =========================================================================}
procedure TfrmVirtualVA.OnUpdateStatus(Sender: TObject; Action: TvvaAction;
Status: TvvaStatus);
begin
  case Ord(Action) of
    0: if Status = stFailed then
       begin  //Validate - set all controls to failed when validation fails
         UpdateStatusDisplay(edFormatText, imgFormatText, Status);
         UpdateStatusDisplay(edTransmitFile, imgTransmitFile, Status);
         UpdateStatusDisplay(edDeleteFile, imgDeleteFile, Status);
       end;
    1: begin //Create
         UpdateStatusDisplay(edFormatText, imgFormatText, Status);
       end;
    2: begin //Xmit
         UpdateStatusDisplay(edTransmitFile, imgTransmitFile, Status);
       end;
    3: begin  //Delete
         UpdateStatusDisplay(edDeleteFile, imgDeleteFile, Status);
       end;
    4: begin  //Complete
         btnClose.Enabled := True;
         btnClose.SetFocus;
       end;
  end;
end;

{=========================================================================
 UpdateStatusDisplay
 This procedure updates the individual components in the status display
 when an action occurs.
 =========================================================================}
procedure TfrmVirtualVA.UpdateStatusDisplay(anEditBox: TEdit;
  aStatusImg: Timage; aStatus: TvvaStatus);
var
  currentStatus: String;
begin
  currentStatus := GetEnumName(TypeInfo(TvvaStatus), integer(aStatus));
  currentStatus := Copy(currentStatus, 3, Length(currentStatus) - 2);

  if (Pos('...', anEditBox.Text) > 0) then
    anEditBox.Text := Copy(anEditBox.Text, 1, Pos('...', anEditBox.Text) - 1) + '...' + currentStatus
  else
    anEditBox.Text := anEditBox.Text + '...' + currentStatus;

  if (aStatus = stSuccess) or
     (aStatus = stFailed) then  //we only have success and failed images
  begin
    aStatusImg.Picture := nil;
    ImgListStatusIcons.GetBitmap(Ord(aStatus), aStatusImg.Picture.Bitmap);
    aStatusImg.Visible := TRUE;
  end;
  Repaint;
end;

end.
