unit spellcheck;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ComCtrls,
  ExtCtrls,
  ORCtrls,
  Main,
  ORFn,
  Gauges;

type
  TfrmSpellcheck = class(TForm)
    Label1: TLabel;
    editWord: TEdit;
    Label2: TLabel;
    ORListBoxSuggestions: TORListBox;
    ButtonSkip: TButton;
    ButtonIgnoreAll: TButton;
    ButtonAddToDictionary: TButton;
    ButtonChange: TButton;
    ButtonChangeAll: TButton;
    ButtonUndoAll: TButton;
    ButtonCancel: TButton;
    ButtonExport: TButton;
    Label3: TLabel;
    Shape2: TShape;
    ButtonStart: TButton;
    Label5: TLabel;
    RichEditTextToCheck: TRichEdit;
    RichEditIgnoreList: TRichEdit;
    Label6: TLabel;
    Label7: TLabel;
    Gauge1: TGauge;
    Button3: TButton;
    TimerGo: TTimer;
    EditChangeTo: TEdit;
    Label8: TLabel;
    RichEditChangeAll: TRichEdit;
    Label4: TLabel;
    procedure FormShow(Sender: TObject);
    procedure ButtonUndoAllClick(Sender: TObject);
    procedure ButtonCancelClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ButtonStartClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonAddToDictionaryClick(Sender: TObject);
    procedure ButtonSkipClick(Sender: TObject);
    procedure ButtonIgnoreAllClick(Sender: TObject);
    procedure ButtonChangeClick(Sender: TObject);
    procedure ButtonChangeAllClick(Sender: TObject);
    procedure ButtonExportClick(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure TimerGoTimer(Sender: TObject);
    procedure ExtractDictionary;
    procedure ORListBoxSuggestionsClick(Sender: TObject);
    procedure ORListBoxSuggestionsDblClick(Sender: TObject);
    procedure EditChangeToChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure editWordChange(Sender: TObject);
  private
    { Private declarations }
    procedure savedictionary;
    procedure refreshdictionary;
    function resetbuttons: boolean;
  public
    { Public declarations }
  end;

var
  frmSpellcheck: TfrmSpellcheck;
  textposspellcheck: longint;
  posstart, posend: longint;
  wordcounter: integer;
  texttocheck: string;
  whichspellcheckline: integer;
  showndone: boolean;
  xSelStart, xSelLength: integer;
  TempDictionary: TStringList;
  DictionaryLookupTables: array[1..27, 1..27] of TStringList;
  DictionaryChanged: Boolean;
  NumberOfErrorsFound: LongInt;

  // Load the dictionary into a TRichEdit then destroy it to save memory
  // Index each word into a table using the 1st 2 chars of the word
  // Sort each table at close when saving and add back to 1 TRichEdit to save
  // in one file
  // A=1
  // B=2
  // C=3
  // D=4
  // E=5
  // F=6
  // G=7
  // H=8
  // I=9
  // J=10
  // K=11
  // L=12
  // M=13
  // N=14
  // O=15
  // P=16
  // Q=17
  // R=18
  // S=19
  // T=20
  // U=21
  // V=22
  // W=23
  // X=24
  // Y=25
  // Z=26
  // All Others become 27,1

implementation

{$R *.DFM}

{------------------------------------------------------------------------------}
{Function to return a Soundex code for a string passed as a parameter }
{------------------------------------------------------------------------------}
function Soundex(Surname: string; UsePrefix: Boolean): string;
const
  {These values are the standard 6 Soundex groups used by the US Federal Census
  where any letter that does not have a group defaults to 0.}
  SoundExTable: array[65..122] of Byte
  { A B C D E F G H I J K L M N O P Q R S T U V W X Y Z [ / ] ^ _ '}
  = (0, 1, 2, 3, 0, 1, 2, 0, 0, 2, 2, 4, 5, 5, 0, 1, 2, 6, 2, 3, 0, 1, 0, 2, 0, 2, 0, 0, 0, 0, 0, 0,
    { a b c d e f g h i j k l m n o p q r s t u v w x y z}
    0, 1, 2, 3, 0, 1, 2, 0, 0, 2, 2, 4, 5, 5, 0, 1, 2, 6, 2, 3, 0, 1, 0, 2, 0, 2);
  MaxCodeLength = 64;
var
  SoundexCode: string;
  ThisChar: Longint;
  LetterPos, NumToAdd, LastNum: Byte;
begin
  if Surname = '' then begin {Exit if nothing passed as parameter}
    MessageDlg('Please enter a term.', mtError, [mbOK], 0);
    Exit;
  end
  else begin
    {Get rid of the longest possible prefix if the Remove prefixes box checked}
    //if RemovePrefix.Checked then
    //Surname := StripPrefix(Surname);
    {Strip white space at start of surname parameter and exit if no text passed}
    repeat
      SoundexCode := UpperCase(Copy(Surname, 1, 1));
      Surname := Copy(Surname, 2, Length(Surname) - 1);
    until (SoundexCode[1] in ['A'..'Z']) or (Length(Surname) = 0);
    {Check to see if string passed is just white space and no letters}
    if (Length(Surname) = 0) and (not (SoundexCode[1] in ['A'..'Z'])) then
      Exit;
    {Generate the numeric code for the first letter of the name}
    LastNum := SoundExTable[Ord(SoundexCode[1])];
    {Iterate the surname string minus its leading letter}
    LetterPos := 1;
    while LetterPos < Length(Surname) + 1 do begin
      ThisChar := Ord(Surname[LetterPos]); {Get ordinal value of next letter}
      case ThisChar of
        {Is character a letter in the range from 'A' to 'z'?}
        65..90, 97..122: begin
            NumToAdd := SoundExTable[ThisChar]; {Read Soundex num in lookup table}
            {Only add it if it is non-zero and different from LastNum}
            if ((NumToAdd > 0) and (NumToAdd <> LastNum)) then begin
              {Augment Soundex code string}
              SoundexCode := SoundexCode + IntToStr(NumToAdd);
              LastNum := NumToAdd; {Set for check of duplicate next time round}
              {Handle repeat letters by leaving the value of LastNum unaltered}
            end
            else
              if (NumToAdd = LastNum) then
                {Do nothing}
              else
                LastNum := 0; {Handle zero value letters}
          end;
        32, 45: {Treat space and hyphen as if letters were contiguous}
          {Do nothing to LastNum value}
          {Is character some other non-letter?}
        else
          LastNum := 0;
      end;
      {Exit the while loop if string reaches max of value set in spinedit}
      if Length(SoundexCode) = maxcodelength then
        break;
      Inc(LetterPos);
    end;
    {Pad Soundex string with trailing 0s if needed}
    while Length(SoundexCode) < maxcodelength do
      SoundexCode := SoundexCode + '0';
    Result := SoundexCode; {Return the character string generated}
  end;
  // strip trailing 0's
  if result <> '' then
    if result[length(result)] = '0' then
      repeat
        result := Copy(result, 1, length(result) - 1);
      until (result[length(result)] <> '0') or (result = '');
end;

procedure TfrmSpellcheck.FormShow(Sender: TObject);
begin
  resetbuttons;
  textposspellcheck := 1;
  TimerGo.Enabled := True;
end;

procedure TfrmSpellcheck.SaveDictionary;
var
  x, y, z: integer;
  st: string;
begin
  screen.cursor := crHourglass;
  TempDictionary := TStringList.Create;

  for x := 1 to 27 do
    for y := 1 to 27 do begin
      if DictionaryLookupTables[x, y].Count > 0 then
        for z := 0 to DictionaryLookupTables[x, y].Count - 1 do
          TempDictionary.Add(DictionaryLookupTables[x, y].Strings[z]);
    end;

  st := tempdir + 'capridictionary.dic';
  try
  TempDictionary.SaveToFile(st)except
    showmessage('Cannot save spellcheck dictionary: ' + st)
  end;

  TempDictionary.Destroy;
  screen.cursor := crDefault;
end;

procedure TfrmSpellcheck.ButtonUndoAllClick(Sender: TObject);
var
  st: string;
begin
  RichEditIgnoreList.Lines.Clear;
  RichEditChangeAll.Lines.Clear;
  // SaveDictionary;

  if DictionaryChanged then begin
    TempDictionary := TStringList.Create;
    st := tempdir + 'capridictionary.dic';
    try
    TempDictionary.LoadFromFile(st)except
      showmessage('Cannot load dictionary for spellcheck: ' + st)
    end;
    RefreshDictionary;
    TempDictionary.Destroy;
  end;

  DictionaryChanged := False;
  resetbuttons;
  ModalResult := mrCancel;
end;

procedure TfrmSpellcheck.ButtonCancelClick(Sender: TObject);
begin
  modalresult := mrOK;
end;

procedure TfrmSpellcheck.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  RichEditIgnoreList.Lines.Clear;
  RichEditChangeAll.Lines.Clear;
  if DictionaryChanged then begin
    SaveDictionary;
  end;

  DictionaryChanged := False;
  resetbuttons;

  if ModalResult <> mrOK then
    ModalResult := mrCancel;

  NumberOfErrorsFound := 0;
  WhichSpellCheckLine := 0;
  ShownDone := False;
  NumberOfErrorsFound := 0;
end;

function TfrmSpellcheck.resetbuttons: Boolean;
begin
  Result:=True;
  EditChangeTo.Text := '';
  ORListBoxSuggestions.Items.Clear;
  buttonskip.enabled := false;
  buttonIgnoreAll.enabled := false;
  buttonAddToDictionary.enabled := false;
  buttonChange.Enabled := false;
  buttonChangeAll.Enabled := false;
  editword.Text := '';
  editword.color := clwhite;
  ORListBoxSuggestions.Items.Clear;
  application.processmessages;
end;

function CheckCase(s: string): integer;
begin
  Result := 0; // Normal;
  if length(s) = 0 then
    exit;
  if Uppercase(s[1]) = s[1] then
    result := 1; // Uppercase 1st char;
  if Uppercase(s) = s then
    result := 2; // All uppercase
end;

procedure TfrmSpellcheck.ButtonStartClick(Sender: TObject);
var
  xx: integer;
  currentword: string;
  currentwordtocheck: string;
  ch: char;
  foundprimary: boolean;
  foundignore: boolean;
  selstart: integer;
  appendchars: string;

  function isnumeral(s: string): boolean;
  var
    x: integer;
  begin
    if length(s) > 0 then begin
      for x := 1 to length(s) do
        if (ord(s[x]) < 48) or (ord(s[x]) > 57) then begin
          result := false;
          exit;
        end;
    end
    else begin
      result := false;
      exit;
    end;
    result := true;
  end;

  function isinlist(s: string): boolean;
  var
    x, y: integer;
    stg: string;
  begin
    result := false;
    // Only evaluate if word is >1 character
    if length(s) = 1 then begin
      result := true;
      exit;
    end;
    s := #13 + #10 + uppercase(s) + '^';
    if length(s) > 4 then begin
      // Word is >1 letter
      stg := uppercase(Copy(s, 3, 1));
      ch := stg[1];
      x := ord(ch) - 64;
      stg := uppercase(Copy(s, 4, 1));
      ch := stg[1];
      y := ord(ch) - 64;
    end
    else begin
      // Word is 1 letter
      stg := uppercase(Copy(s, 3, 1));
      ch := stg[1];
      x := ord(ch) - 64;
      y := 1;
    end;

    if (x < 0) or (y < 0) or (x > 27) or (y > 27) then begin
      x := 27;
      y := 1;
    end;

    if (Pos(s, DictionaryLookupTables[x, y].Text) > 0) or
      (Copy(DictionaryLookupTables[x, y].Text, 1, length(s) - 2) = Copy(s, 3, length(s) - 2))
      then
      result := true;
  end;

  function isinignorelist(s: string): boolean;
  var
    x: integer;
  begin
    result := false;
    if RichEditIgnoreList.Lines.Count > 0 then
      for x := 0 to RichEditIgnoreList.Lines.Count - 1 do
        if RichEditIgnoreList.Lines[x] = s + '^' then
          result := true;
  end;

  procedure findclosematches(s: string);
  var
    soundex1: string;
    x, y, z: integer;
    stg: string;
    ch: char;

  begin
    ORListBoxSuggestions.Visible := False;

    soundex1 := '^' + SoundEx(s, false);
    screen.cursor := crHourglass;

    if length(s) > 1 then begin
      // Word is >1 letter
      stg := uppercase(Copy(s, 1, 1));
      ch := stg[1];
      x := ord(ch) - 64;
      stg := uppercase(Copy(s, 2, 1));
      ch := stg[1];
      y := ord(ch) - 64;
    end
    else begin
      // Word is 1 letter
      stg := uppercase(Copy(s, 1, 1));
      ch := stg[1];
      x := ord(ch) - 64;
      y := 1;
    end;
    if (x < 0) or (y < 0) or (x > 27) or (y > 27) then begin
      x := 27;
      y := 1;
    end;

    if DictionaryLookupTables[x, y].Count > 0 then
      for z := 0 to DictionaryLookupTables[x, y].Count - 1 do begin
        if Pos(soundex1, DictionaryLookupTables[x, y].Strings[z]) > 0 then begin
          s := lowercase(Piece(DictionaryLookupTables[x, y].Strings[z], '^', 1));
          //If CheckCase(currentword)=0 then s:=s; // Do nothing
          if CheckCase(currentwordtocheck) = 1 then
            s := uppercase(string(s[1])) + copy(s, 2, length(s) - 1)
          else // First letter capitalized
            if CheckCase(currentwordtocheck) = 2 then
              s := uppercase(s); // All caps
          // Make sure it doesn't already have an apostrophe in the word
          // Screen out offensive terms
          if ((uppercase(s) <> 'NIGGER') and (uppercase(s) <> 'NIGGERDOM') and
            (uppercase(s) <> 'NIGGERED') and (uppercase(s) <> 'NIGGERS') and
            (uppercase(s) <> 'NIGGERLING') and (uppercase(s) <> 'NIGGERISH') and
            (uppercase(s) <> 'NIGGERISM') and (uppercase(s) <> 'NIGGERTOE') and
            (uppercase(s) <> 'NIGGERY') and (uppercase(s) <> 'NIGGERGOOSE') and
            (uppercase(s) <> 'NIGGERFISH') and (uppercase(s) <> 'NIGGERFISHES') and
            (uppercase(s) <> 'NIGGERHEAD') and (uppercase(s) <> 'NIGGERWEED')) then begin
            if pos('''', s) = 0 then
              ORListBoxSuggestions.Items.Add(s + appendchars)
            else
              ORListBoxSuggestions.Items.Add(s);
          end;
          ORListBoxSuggestions.Repaint;
        end;
      end;
    if ORListBoxSuggestions.Items.Count = 0 then begin
      ORListBoxSuggestions.Items.Add('< No suggestions found. >');
    end;

    application.processmessages;
    screen.cursor := crDefault;
    ORListBoxSuggestions.Visible := True;
    //buttonignoreallclick(application);
  end;

begin
  // Rules
  // When adding a new value, ask user if capitalization matters (for acroynms, proper names, etc.)
  // If a dash after a word and then followed by a character other than a space, it's all one word
  // If period at the end of a word then followed by a character other than a space, it's all one word
  // If a dash at the end followed by a space, remove the dash from the word
  // If a period at the end followed by a space, remove the period
  // If a numeral, then ignore it
  // If encountering a comma, consider it as the end of a word.
  // If encountering a non-alphabetic or numeric character, consider it as a space
  // Underscore is the same as a dash
  // Two dashes are a hyphen, so don't link the words together
  // If 's at the end of the word, drop the 's before checking

  // convert word to lowercase when matching
  // if no match, don't convert and try to match - might be a user exact match addition

  (*
  SelStart := SendMessage(RichEditTextToCheck.Handle,EM_LINEINDEX,WhichSpellcheckLine,0);
  RichEditTextToCheck.SelStart := SelStart;
  RichEditTextToCheck.SelLength := length(RichEditTextToCheck.Lines[WhichSpellCheckLine]);
  // Force text to jump down
  RichEditTextToCheck.SelText:=RichEditTextToCheck.Lines[WhichSpellCheckLine];
  RichEditTextToCheck.SetFocus;
  // Highlight text
  RichEditTextToCheck.SelStart := SelStart;
  RichEditTextToCheck.SelLength := length(RichEditTextToCheck.Lines[WhichSpellCheckLine]);
  RichEditTextToCheck.Repaint;
  RichEditTextToCheck.SetFocus;
*)

  xx := textposspellcheck;
  posstart := xx;
  currentword := '';
  currentwordtocheck := '';
  appendchars := '';
  if Length(TextToCheck) > 0 then
    repeat
      // allow last word to be processed
      if xx <= Length(TextToCheck) then
        ch := TextToCheck[xx]
      else
        ch := #0;
      // 48 to 57 = numerals
      // 65 - 90 = uppercase
      // 97 - 122 = lowercase
      // Double dashes, periods or underscores end a word
      if ((ch = '-') or (ch = '_')) and (currentword <> '') then
        if (currentword[length(currentword)] = '-') or (currentword[length(currentword)] = '_') then
          ch := #0;

      if ch = '_' then
        ch := ' '; // Treat underscores like a space

      if ((ord(ch) > 47) and (ord(ch) < 58)) or ((ord(ch) > 64) and (ord(ch) < 91)) or
        //((ord(ch)>96) and (ord(ch)<123)) or (ch='-') or (ch='_') or (ch='''')
      ((ord(ch) > 96) and (ord(ch) < 123)) or (ch = '_') or (ch = '''')
        then begin
        // Deal with compound words as a single word
        if (ch = '-') and (currentword = '') then
          ch := #0;
        if ch <> #0 then
          currentword := currentword + ch;
      end
      else begin
        if currentword <> '' then begin
          inc(wordcounter);
          // Check if apostrophe at end or apostrophe + s
          currentwordtocheck := currentword;
          if length(currentword) > 3 then
            if currentword[length(currentword) - 1] = '''' then
              if currentword[length(currentword)] = 's' then begin
                currentwordtocheck := copy(currentword, 1, length(currentword) - 2);
                appendchars := copy(currentword, length(currentword) - 1, 2);
              end;
          if length(currentword) > 1 then
            if currentword[length(currentword)] = '''' then begin
              currentwordtocheck := copy(currentword, 1, length(currentword) - 1);
              appendchars := '''';
            end;

          // RichEditShortList.Lines.SaveToFile(tempdir+'capridictionaryshortlist.dic');
          //if wordcounter/50=trunc(wordcounter/50) then
            //frmSpellCheck.Caption:=inttostr(wordcounter);

          if isnumeral(CurrentWordToCheck) = false then begin
            // make sure it isn't a number
            posend := xx;
            // strip off final period, dash or underscore, if it exists.
            if (CurrentWordToCheck[length(CurrentWordToCheck)] = '-') or (CurrentWordToCheck[length(CurrentWordToCheck)] = '_') then begin
              CurrentWordToCheck := copy(CurrentWordToCheck, 1, length(CurrentWordToCheck) - 1);
              posend := posend - 1;
            end;
            // Highlight word here

            foundignore := isinIgnoreList(Uppercase(CurrentWordToCheck));

            if foundignore = false
              then begin
              foundprimary := isinlist(CurrentWordToCheck);
              if foundprimary = false
                then begin
                RichEditTextToCheck.SetFocus;
                RichEditTextToCheck.Lines[WhichSpellcheckLine] := RichEditTextToCheck.Lines[WhichSpellcheckLine];
                SelStart := SendMessage(RichEditTextToCheck.Handle, EM_LINEINDEX, WhichSpellcheckLine, 0);
                xSelStart := SelStart + xx - length(currentword) - 1;
                xSelLength := length(CurrentWord);
                RichEditTextToCheck.SelStart := xSelStart;
                RichEditTextToCheck.SelLength := xSelLength;
                // Force text to jump down
                RichEditTextToCheck.SelText := CurrentWord;
                RichEditTextToCheck.SetFocus;
                // Highlight text
                RichEditTextToCheck.SelStart := SelStart + xx - length(currentword) - 1;
                RichEditTextToCheck.SelLength := length(CurrentWord);
                RichEditTextToCheck.Repaint;
                RichEditTextToCheck.SetFocus;

                buttonaddtodictionary.enabled := true;
                buttonskip.enabled := true;
                buttonIgnoreAll.enabled := true;
                textposspellcheck := xx + 1;
                posstart := xx;
                posend := -1;
                ORListBoxSuggestions.Items.Clear;
                try
                findclosematches(currentword)except
                end;
                editword.text := currentwordtocheck + appendchars;
                editword.color := clYellow;
                //try editChangeTo.Setfocus except end;
                exit;
              end
              else begin
                posstart := xx + 1;
              end;
            end
            else begin
              // Word is in shortlist already
              posstart := xx + 1;
            end;
          end
          else begin
            // Word is a numeral
            posstart := xx + 1;
          end;
          currentword := '';
          currentwordtocheck := '';
          appendchars := '';
        end;
      end;
      inc(xx);
    until xx > Length(TextToCheck) + 1;

  if ButtonSkip.Enabled = False then
    textposspellcheck := 1;
  WordCounter := 0;
end;

procedure TfrmSpellcheck.RefreshDictionary;
var
  x, y, z: integer;
  ch: char;
  stg: string;
begin
  // Process dictionary here and split it all up into pieces
  if tempDictionary.Count = 0 then begin
    Showmessage('The dictionary is empty.');
    modalresult := mrCancel;
    exit;
  end;

  for x := 1 to 27 do
    for y := 1 to 27 do
      DictionaryLookupTables[x, y].Clear;

  for z := 0 to tempDictionary.Count - 1 do
    if length(piece(tempDictionary[z], '^', 1)) > 1 then begin
      // Word is >1 letter
      stg := uppercase(Copy(tempDictionary[z], 1, 1));
      ch := stg[1];
      x := ord(ch) - 64;
      stg := uppercase(Copy(tempDictionary[z], 2, 1));
      ch := stg[1];
      y := ord(ch) - 64;
      if (x < 0) or (y < 0) or (x > 27) or (y > 27) then begin
        x := 27;
        y := 1;
      end;
      DictionaryLookupTables[x, y].Add(uppercase(tempDictionary[z]));
    end
    else
      if length(piece(tempDictionary[z], '^', 1)) = 1 then begin
        // Word is 1 letter
        stg := uppercase(Copy(tempDictionary[z], 1, 1));
        ch := stg[1];
        x := ord(ch) - 64;
        y := 1;
        if (x < 0) or (y < 0) or (x > 27) or (y > 27) then begin
          x := 27;
          y := 1;
        end;
        DictionaryLookupTables[x, y].Add(uppercase(tempDictionary[z]));
      end;
end;

procedure TfrmSpellcheck.FormCreate(Sender: TObject);
var
  st: string;
  x, y: integer;
begin
  NumberOfErrorsFound := 0;

  TempDictionary := TStringList.Create;
  for x := 1 to 27 do
    for y := 1 to 27 do begin
      DictionaryLookupTables[x, y] := TStringList.Create;
      DictionaryLookupTables[x, y].Add('');
    end;
  // Check if dictionary has been saved to disc from compile resource
  if FileExists(tempdir + 'capridictionary.dic') = False then
    extractdictionary;

  WhichSpellCheckLine := 0;
  ShownDone := False;

  st := tempdir + 'capridictionary.dic';
  try
  TempDictionary.LoadFromFile(st)except
    showmessage('Cannot load dictionary for spellcheck: ' + st)
  end;

  RefreshDictionary;

  // Now destroy the dictionary to save memory
  TempDictionary.Destroy;

end;

procedure TfrmSpellcheck.ButtonAddToDictionaryClick(Sender: TObject);
var
  s: string;
  ch: char;
  stg: string;
  x, y: integer;
begin
  s := editword.text;
  x := 0;
  y := 0;
  if length(s) > 1 then begin
    // Word is >1 letter
    stg := uppercase(Copy(s, 1, 1));
    ch := stg[1];
    x := ord(ch) - 64;
    stg := uppercase(Copy(s, 2, 1));
    ch := stg[1];
    y := ord(ch) - 64;
  end
  else
    if length(s) = 1 then begin
      // Word is 1 letter
      stg := uppercase(Copy(s, 1, 1));
      ch := stg[1];
      x := ord(ch) - 64;
      y := 1;
    end;
  if (x < 0) or (y < 0) or (x > 27) or (y > 27) then begin
    x := 27;
    y := 1;
  end;
  DictionaryLookupTables[x, y].Add(Uppercase(editword.text) + '^' + SoundEx(editword.text, false));

  DictionaryChanged := True;

  ButtonUndoAll.Enabled := True;
  resetbuttons;
  Button3Click(application);
end;

procedure TfrmSpellcheck.ButtonSkipClick(Sender: TObject);
begin
  //textposspellcheck:=textposspellcheck+length(editWord.Text);
  resetbuttons;
  Button3Click(application);
end;

procedure TfrmSpellcheck.ButtonIgnoreAllClick(Sender: TObject);
begin
  // Make sure at least one blank line at the start
  if RichEditIgnoreList.Lines.Count = 0 then
    RichEditIgnoreList.Lines.Add('');
  RichEditIgnoreList.Lines.Add(uppercase(editWord.text) + '^');
  resetbuttons;
  Button3Click(application);
end;

procedure TfrmSpellcheck.ButtonChangeClick(Sender: TObject);
begin

  RichEditTextToCheck.SelStart := xSelStart;
  RichEditTextToCheck.SelLength := xSelLength;
  textposspellcheck := textposspellcheck - length(editword.text) - 1;
  RichEditTextToCheck.SelText := EditChangeTo.Text;

  resetbuttons;
  ButtonUndoAll.Enabled := True;
  Button3Click(application);
end;

procedure TfrmSpellcheck.ButtonChangeAllClick(Sender: TObject);
begin
  RichEditChangeAll.Lines.Add(editWord.Text + '^' + EditChangeTo.Text);

  RichEditTextToCheck.SelStart := xSelStart;
  RichEditTextToCheck.SelLength := xSelLength;
  textposspellcheck := textposspellcheck - length(editword.text) - 1;
  RichEditTextToCheck.SelText := EditChangeTo.Text;

  resetbuttons;

  ButtonUndoAll.Enabled := True;
  Button3Click(application);
end;

procedure TfrmSpellcheck.ButtonExportClick(Sender: TObject);
begin
  if Application.MessageBox('This will restore the dictionary back to the original and lose all of your customized additions.  Do you want to do this?', 'Reset Dictionary?', 4) = 6 then begin
    try
    DeleteFile(tempdir + 'capridictionary.dic')except
    end;
    TempDictionary := TStringList.Create;
    ExtractDictionary;
    RefreshDictionary;
    TempDictionary.Destroy;
    RichEditIgnoreList.Clear;
  end;
end;

procedure TfrmSpellcheck.Button3Click(Sender: TObject);
var
  xx: integer;
begin

  editchangeto.enabled := true;

  // RichEditIgnoreList.Lines.Clear;
  TextToCheck := '';
  if ShownDone = True then
    exit;
  if RichEditTextToCheck.Lines.Count > 0 then begin
    Gauge1.MaxValue := RichEditTextToCheck.Lines.Count - 1;
    for xx := whichspellcheckline to RichEditTextToCheck.Lines.Count - 1 do begin
      Gauge1.Progress := xx;
      //Gauge1.Repaint;
      TextToCheck := RichEditTextToCheck.Lines[xx];
      whichSpellcheckLine := xx;
      ButtonStartClick(Application);
      if ButtonSkip.Enabled = True then
        exit;
    end;
    if ShownDone = True then
      exit;
    if NumberOfErrorsFound = 0 then begin
      Showmessage('No errors found.');
      modalresult := mrCancel;
      exit;
    end;
    Showmessage('Done.');
    ShownDone := True;
    Gauge1.Progress := 0;
    exit;
  end
  else begin
    Showmessage('There is nothing to check.');
    modalresult := mrOK;
  end;
  Gauge1.Progress := 0;

  editchangeto.enabled := false;
end;

procedure TfrmSpellcheck.TimerGoTimer(Sender: TObject);
begin
  TimerGo.Enabled := False;
  Button3Click(Application);
end;

procedure TfrmSpellcheck.ExtractDictionary;
var
  rStream: TResourceStream;
  fStream: TMemoryStream;
begin
  screen.cursor := crHourglass;
  rStream := TResourceStream.Create(hInstance, 'capridictionary', RT_RCDATA);
  try
    fStream := TMemoryStream.Create;
    try
      fStream.CopyFrom(rStream, 0);
      fStream.Position := 0;
      TempDictionary.LoadFromStream(fStream);
      try
      TempDictionary.SaveToFile(tempdir + 'capridictionary.dic')except begin
          Showmessage('Cannot create dictionary for spellcheck.');
          exit;
        end;
      end;
    finally fStream.Free;
    end;
  finally rStream.Free;
  end;
  screen.cursor := crDefault;
end;

procedure TfrmSpellcheck.ORListBoxSuggestionsClick(Sender: TObject);
begin
  if ORListBoxSuggestions.Items[ORListBoxSuggestions.ItemIndex] <> '< No suggestions found. >' then
    EditChangeTo.Text := ORListBoxSuggestions.Items[ORListBoxSuggestions.ItemIndex];
end;

procedure TfrmSpellcheck.ORListBoxSuggestionsDblClick(Sender: TObject);
begin
  if ORListBoxSuggestions.Items[ORListBoxSuggestions.ItemIndex] <> '< No suggestions found. >' then
    ButtonChangeClick(Application);
end;

procedure TfrmSpellcheck.EditChangeToChange(Sender: TObject);
begin
  if EditChangeTo.Text <> '' then begin
    ButtonChange.Enabled := True;
    ButtonChangeAll.Enabled := True;
  end
  else begin
    ButtonChange.Enabled := False;
    ButtonChangeAll.Enabled := False;
  end;
end;

procedure TfrmSpellcheck.FormDestroy(Sender: TObject);
var
  x, y: integer;
begin
  for x := 1 to 27 do
    for y := 1 to 27 do
      DictionaryLookupTables[x, y].Destroy;
end;

procedure TfrmSpellcheck.editWordChange(Sender: TObject);
var
  x: integer;
  flag: boolean;
  s: string;
begin
  flag:=false;
  if editWord.Text <> '' then
    inc(NumberOfErrorsFound);

  s := '';
  if RichEditChangeAll.Lines.Count > 0 then
    for x := 0 to RichEditChangeAll.Lines.Count - 1 do
      // check for exact match
      if editWord.Text = Piece(RichEditChangeAll.Lines[x], '^', 1) then begin
        s := Piece(RichEditChangeAll.Lines[x], '^', 2);
        flag := true;
      end
      else
        // check for match regardless of case
        if uppercase(editWord.Text) = uppercase(Piece(RichEditChangeAll.Lines[x], '^', 1)) then begin
          s := Piece(RichEditChangeAll.Lines[x], '^', 2);
          if CheckCase(editWord.Text) = 1 then
            s := uppercase(string(s[1])) + lowercase(copy(s, 2, length(s) - 1))
          else // First letter capitalized
            if CheckCase(editWord.Text) = 2 then
              s := uppercase(s); // All caps
          flag := true;
        end;
  if flag = true then begin
    EditChangeTo.Text := s;
    buttonchangeclick(Application);
  end;
end;

end.
